
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */

package chapter11;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.DatabaseMetaData;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;

import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.ArrayList;


public class DBInfo {

private DatabaseMetaData dbMetaData;

public static void main( String[] args )  {

String dataSourceName = null;

if ( args.length == 1 ) { // DataSourceName on the command line 
    dataSourceName = args[0];
}
else {
    dataSourceName = "movies";
} 

// create object and get connected 
DBInfo dbinfo = new DBInfo( dataSourceName );


// general db info
dbinfo.output( "General Database Information", dbinfo.getGeneralInfo() );


// tables in database 
dbinfo.output( "Database Tables", dbinfo.getTables() );


// indices in tables 
dbinfo.output( "Table Indices", dbinfo.getIndices( dbinfo.getTables() ) );

// numeric functions
dbinfo.output( "Numeric Functions", dbinfo.getNumFunctions( ) );

// string functions
dbinfo.output( "String Functions", dbinfo.getStrFunctions( ) );

// system functions
dbinfo.output( "System Functions", dbinfo.getSysFunctions( ) );

}

// --------------------------------------------------------------------
public DBInfo( String dataSourceName ) {

getConnected( dataSourceName );

}


// --------------------------------------------------------------------
private void getConnected( String DataSourceName ) {

Connection con;
try {

InitialContext ctx = new InitialContext();

DataSource ds = (DataSource) ctx.lookup( DataSourceName );

con = ds.getConnection();

dbMetaData = con.getMetaData();

}
catch (NamingException e) {
      System.out.println("NamingException in getConnected: " + e.getMessage() );
}
catch (SQLException e) {
      System.out.println("SQLException in getConnected: " + e.getMessage() );
}


}

// --------------------------------------------------------------------
public Collection getTables() {
ArrayList retList = new ArrayList();

try {

ResultSet rs = dbMetaData.getTables( null,  // catalog 
                          null,  // schema 
			  null,  // table name pattern 
			  null ); // table types to include 

while ( rs.next() ) {
        retList.add( "Table Name");
	retList.add( rs.getString( "TABLE_NAME" ));
	retList.add( "Table Type" ); 
	retList.add( rs.getString( "TABLE_TYPE" ));
}

}
catch (SQLException e) {
      System.out.println("SQLException in getTables: " + e.getMessage() );
}
finally { 
   return retList;
}

}

// --------------------------------------------------------------------
public Collection getIndices( Collection tables ) {

String tableName = null;
ArrayList retList = new ArrayList();

// need the positioning capabilities of the arrayList 
ArrayList tablesList = (ArrayList) tables;
int position = 1;                 // start at 1 and skip the first label 
int size     = tablesList.size();

try { 

// for each table 
while ( position < size ) {

tableName = (String) tablesList.get( position );
position += 4; // move past the table type info and labels

ResultSet rs = dbMetaData.getIndexInfo( null,    // catalog 
                                        null,    // schema 
			                tableName,    // table name pattern 
			                false,   // unique
					true);  // approximate

while ( rs.next() ) {
       retList.add( "Table Name" );
       retList.add( rs.getString( "TABLE_NAME" ));
       retList.add( "Index Name" ); 
       retList.add( rs.getString( "INDEX_NAME" ));
       retList.add( "Column Name" ); 
       retList.add( rs.getString( "COLUMN_NAME" ));
}

} // table names loop

}
catch (SQLException e) {
       System.out.println("SQLException in getIndices: " + e.getMessage() );
}
finally {
   return retList;
}

}

// --------------------------------------------------------------------
public Collection getGeneralInfo() {

ArrayList retList = new ArrayList();

try { 

// info goes into the set in pairs - label-value
retList.add( "Database Product" );
retList.add( dbMetaData.getDatabaseProductName() );

retList.add( "Database Version" );
retList.add( dbMetaData.getDatabaseProductVersion() );

retList.add( "JDBC Driver" );
retList.add( dbMetaData.getDriverVersion() );

retList.add( "DB URL" );
retList.add( dbMetaData.getURL() );

retList.add( "Maximum Open Statements" );
retList.add( new Integer( dbMetaData.getMaxStatements())  );

retList.add( "Maximum Open Connections" );
retList.add( new Integer( dbMetaData.getMaxConnections()) );

retList.add( "Maximum Procedure Name Length" );
retList.add( new Integer( dbMetaData.getMaxProcedureNameLength())  );

retList.add( "Maximum Schema Name Length" );
retList.add( new Integer( dbMetaData.getMaxSchemaNameLength()) );

retList.add( "Maximum Statement Length" );
retList.add( new Integer( dbMetaData.getMaxStatementLength()) );

retList.add( "Maximum Table Name Length" );
retList.add( new Integer( dbMetaData.getMaxTableNameLength()) );

retList.add( "Maximum User Name Length" );
retList.add( new Integer( dbMetaData.getMaxUserNameLength()) );

retList.add( "Maximum Cursor Name Length" );
retList.add( new Integer( dbMetaData.getMaxCursorNameLength()) );

retList.add( "Is a Read Only Database" );
retList.add( (dbMetaData.isReadOnly() ? "TRUE" : "FALSE") );

retList.add( "Connected User Name" );
retList.add( dbMetaData.getUserName() );

retList.add( "Supports Non-Null Columns" );
retList.add( (dbMetaData.supportsNonNullableColumns() ? "TRUE" : "FALSE") );

retList.add( "Uses Local Files" );
retList.add( (dbMetaData.usesLocalFiles() ? "TRUE" : "FALSE") );

retList.add( "Uses Local Files Per Table" );
retList.add( (dbMetaData.usesLocalFilePerTable() ? "TRUE" : "FALSE") );

}
catch (SQLException e) {
      System.out.println( "SQLException in getGeneralInfo: " + e.getMessage() );
}
finally {
      return retList;
}


}
// --------------------------------------------------------------------

public Collection getNumFunctions() {

ArrayList retList = new ArrayList();

try { 

String numericFunctions = dbMetaData.getNumericFunctions(); 
StringTokenizer stringParser = new StringTokenizer( numericFunctions, // String to parse
                                                    ",",              // token delimiter 
						    false  );         // don't return the token

// parse the string into a collection 
while ( stringParser.hasMoreTokens() ) {
       retList.add( "Function Name" );
       retList.add( stringParser.nextToken() ); 
}

}
catch (SQLException e) {
       System.out.println("SQLException caught in getNumFunctions: " + e.getMessage() );
}

finally {
  return retList;
}

}

// --------------------------------------------------------------------

public Collection getSysFunctions() {

ArrayList retList = new ArrayList();

try { 

String systemFunctions = dbMetaData.getSystemFunctions(); 
StringTokenizer stringParser = new StringTokenizer( systemFunctions, // String to parse
                                                    ",",              // token delimiter 
						    false  );         // don't return the token

// parse the string into a collection 
while ( stringParser.hasMoreTokens() ) {
       retList.add( "Function Name" );
       retList.add( stringParser.nextToken() ); 
}

}
catch (SQLException e) {
       System.out.println("SQLException caught in getSysFunctions: " + e.getMessage() );
}

finally {
  return retList;
}

}

// --------------------------------------------------------------------
public Collection getStrFunctions() {

ArrayList retList = new ArrayList();

try { 

String stringFunctions = dbMetaData.getStringFunctions(); 
StringTokenizer stringParser = new StringTokenizer( stringFunctions, // String to parse
                                                    ",",              // token delimiter 
						    false  );         // don't return the token

// parse the string into a collection 
while ( stringParser.hasMoreTokens() ) {
       retList.add( "Function Name" );
       retList.add( stringParser.nextToken() ); 
}

}
catch (SQLException e) {
       System.out.println("SQLException caught in getStrFunctions: " + e.getMessage() );
}

finally {
  return retList;
}

}



// --------------------------------------------------------------------
public void output( String title, Collection outputCollection ) {

Iterator i = outputCollection.iterator();

System.out.println( "*********************************" );
System.out.println( "<<<< " + title + ">>>>" );
System.out.println( "*********************************" );
while ( i.hasNext() ) {
       System.out.println( i.next() + "\t" +   // the label 
                           i.next()  );  // the value 
}
System.out.println( "*********************************" );

}

}
