/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */

package chapter11;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.DatabaseMetaData;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;


public class DBInfoGUI {

private DatabaseMetaData dbMetaData;

public static void main( String[] args )  {

String dataSourceName = null;
ArrayList panelInfo = new ArrayList();

if ( args.length == 1 ) { // DataSourceName on the command line 
    dataSourceName = args[0];
}
else {
    dataSourceName = "movies";
} 

// create object and get connected 
DBInfoGUI dbinfo = new DBInfoGUI( dataSourceName );

// general db info
System.out.println( "Loading database metadata ... " );
panelInfo.add( "General Information");
panelInfo.add( dbinfo.getGeneralInfo() );

panelInfo.add( "Database Tables" );
panelInfo.add( dbinfo.getTables() );

panelInfo.add( "Table Indices" );
panelInfo.add( dbinfo.getIndices( (ArrayList) dbinfo.getTables() ) );

panelInfo.add( "Numeric Functions" );
panelInfo.add( dbinfo.getNumFunctions() );

panelInfo.add( "String Functions" );
panelInfo.add( dbinfo.getStrFunctions() );

panelInfo.add( "System Functions" );
panelInfo.add( dbinfo.getSysFunctions() );

panelInfo.add( "SQL Syntax and Limitations" );
panelInfo.add( dbinfo.getSQLSyntax() );

// build the GUI and display
System.out.println( "Building and displaying GUI ... " );
dbinfo.buildGUI( panelInfo );

}

// --------------------------------------------------------------------
public DBInfoGUI( String dataSourceName ) {

getConnected( dataSourceName );

}

// --------------------------------------------------------------------

private void buildGUI( ArrayList panelInfo ) {

JFrame frame = new JFrame("Database Information");
JPanel  contentPanel = new JPanel(new BorderLayout());
frame.setContentPane(contentPanel);

JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.TOP );

int position = 1;

// load the panels with the information we've collected 
while ( position < panelInfo.size() ) {

   // every 1st element is the name of the info stored
   // every 2nd element is the ArrayList with info 

   JPanel panel = loadPanel( (ArrayList) panelInfo.get( position ) ); 
   tabbedPane.add( panel, (String) panelInfo.get( (position - 1) ) );      

   position += 2;

}


//
// set the window closing event
//
frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
           System.exit(0);
      }
    });

// add the tabbed pane to the frame's content panel
contentPanel.add( tabbedPane );

// turn it on 
frame.pack();
frame.setVisible(true);


}

// --------------------------------------------------------------------
private void getConnected( String DataSourceName ) {

Connection con;
try {

InitialContext ctx = new InitialContext();

DataSource ds = (DataSource) ctx.lookup( DataSourceName );

con = ds.getConnection();

dbMetaData = con.getMetaData();

}
catch (NamingException e) {
      System.out.println("NamingException in getConnected: " + e.getMessage() );
}
catch (SQLException e) {
      System.out.println("SQLException in getConnected: " + e.getMessage() );
}


}

// --------------------------------------------------------------------
public Collection getTables() {
ArrayList retList = new ArrayList();

try {

ResultSet rs = dbMetaData.getTables( null,  // catalog 
                          null,  // schema 
			  null,  // table name pattern 
			  null ); // table types to include 

while ( rs.next() ) {
        retList.add( "Table Name");
	retList.add( rs.getString( "TABLE_NAME" ));
	retList.add( "Table Type" ); 
	retList.add( rs.getString( "TABLE_TYPE" ));
}

}
catch (SQLException e) {
      System.out.println("SQLException in getTables: " + e.getMessage() );
}
finally { 
   return retList;
}

}

// --------------------------------------------------------------------
public Collection getIndices( ArrayList tables ) {

String tableName  = null;
ArrayList retList = new ArrayList();

// need the positioning capabilities of the arrayList 
int position = 1;                 // start at 1 and skip the first label 
int size     = tables.size();

try { 

// for each table 
while ( position < size ) {

tableName = (String) tables.get( position );
position += 4; // move past the table type info and labels

ResultSet rs = dbMetaData.getIndexInfo( null,    // catalog 
                                        null,    // schema 
			                tableName,    // table name pattern 
			                false,   // unique
					true);  // approximate

while ( rs.next() ) {
       retList.add( "Table Name" );
       retList.add( rs.getString( "TABLE_NAME" ));
       retList.add( "Index Name" ); 
       retList.add( rs.getString( "INDEX_NAME" ));
       retList.add( "Column Name" ); 
       retList.add( rs.getString( "COLUMN_NAME" ));
}

} // table names loop

}
catch (SQLException e) {
       System.out.println("SQLException in getIndices: " + e.getMessage() );
}
finally {
   return retList;
}

}

// --------------------------------------------------------------------
public Collection getGeneralInfo() {

ArrayList retList = new ArrayList();

try { 

// info goes into the set in pairs - label-value
retList.add( "Database Product" );
retList.add( dbMetaData.getDatabaseProductName() );

retList.add( "Database Version" );
retList.add( dbMetaData.getDatabaseProductVersion() );

retList.add( "JDBC Driver Version" );
retList.add( dbMetaData.getDriverVersion() );

retList.add( "JDBC Driver Name" );
retList.add( dbMetaData.getDriverName() );

retList.add( "DB URL" );
retList.add( dbMetaData.getURL() );

retList.add( "Maximum Open Statements" );
retList.add( new Integer( dbMetaData.getMaxStatements())  );

retList.add( "Maximum Open Connections" );
retList.add( new Integer( dbMetaData.getMaxConnections()) );

retList.add( "Maximum Procedure Name Length" );
retList.add( new Integer( dbMetaData.getMaxProcedureNameLength())  );

retList.add( "Maximum Schema Name Length" );
retList.add( new Integer( dbMetaData.getMaxSchemaNameLength()) );

retList.add( "Maximum Statement Length" );
retList.add( new Integer( dbMetaData.getMaxStatementLength()) );

retList.add( "Maximum Table Name Length" );
retList.add( new Integer( dbMetaData.getMaxTableNameLength()) );

retList.add( "Maximum User Name Length" );
retList.add( new Integer( dbMetaData.getMaxUserNameLength()) );

retList.add( "Maximum Cursor Name Length" );
retList.add( new Integer( dbMetaData.getMaxCursorNameLength()) );

retList.add( "Is a Read Only Database" );
retList.add( (dbMetaData.isReadOnly() ? "TRUE" : "FALSE") );

retList.add( "Connected User Name" );
retList.add( dbMetaData.getUserName() );

retList.add( "Supports Non-Null Columns" );
retList.add( (dbMetaData.supportsNonNullableColumns() ? "TRUE" : "FALSE") );

retList.add( "Uses Local Files" );
retList.add( (dbMetaData.usesLocalFiles() ? "TRUE" : "FALSE") );

retList.add( "Uses Local Files Per Table" );
retList.add( (dbMetaData.usesLocalFilePerTable() ? "TRUE" : "FALSE") );

}
catch (SQLException e) {
      System.out.println( "SQLException in getGeneralInfo: " + e.getMessage() );
}
finally {
      return retList;
}


}
// --------------------------------------------------------------------

public Collection getNumFunctions() {
Collection coll = null;

try {

   coll =  parseFunctionsFromString( dbMetaData.getNumericFunctions() );

}
catch (SQLException e) {
      System.out.println( "SQLException in getGeneralInfo: " + e.getMessage() );
}
finally {
    return coll;
}

}

// --------------------------------------------------------------------


private Collection parseFunctionsFromString( String functionList ) {

ArrayList retList = new ArrayList();

StringTokenizer stringParser = new StringTokenizer( functionList, // String to parse
                                                    ",",              // token delimiter 
						    false  );         // don't return the token

// parse the string into a collection 
while ( stringParser.hasMoreTokens() ) {
       retList.add( "Function Name" );
       retList.add( stringParser.nextToken() ); 
}

  return retList;

}


// --------------------------------------------------------------------


public Collection getSQLSyntax() {

ArrayList retList = new ArrayList();

try { 

String numericFunctions = dbMetaData.getNumericFunctions(); 

retList.add( "Extra Name Characters" );
retList.add( dbMetaData.getExtraNameCharacters());

retList.add( "Search String Escape" );
retList.add( dbMetaData.getSearchStringEscape() );

retList.add( "Identifier Quote String" );
retList.add( dbMetaData.getIdentifierQuoteString() );

retList.add( "Max Binary Literal Length" );
retList.add( new Integer( dbMetaData.getMaxBinaryLiteralLength()) );

retList.add( "Max Catalog Name Length" );
retList.add( new Integer( dbMetaData.getMaxCatalogNameLength()) );

retList.add( "Max Character Literal Length" );
retList.add( new Integer( dbMetaData.getMaxCharLiteralLength()) );

retList.add( "Max Columns in group by" );
retList.add( new Integer( dbMetaData.getMaxColumnsInGroupBy()) );

retList.add( "Max Columns in Index" );
retList.add( new Integer( dbMetaData.getMaxColumnsInIndex()) );

retList.add( "Max Columns in order by" );
retList.add( new Integer( dbMetaData.getMaxColumnsInOrderBy()) );

retList.add( "Max Columns in select" );
retList.add( new Integer( dbMetaData.getMaxColumnsInSelect()) );

retList.add( "Max Tables in select" );
retList.add( new Integer( dbMetaData.getMaxTablesInSelect()) );


}
catch (SQLException e) {
       System.out.println("SQLException caught in getNumFunctions: " + e.getMessage() );
}

finally {
  return retList;
}

}




// --------------------------------------------------------------------

public Collection getSysFunctions() {
Collection coll = null;

try {
     coll = parseFunctionsFromString( dbMetaData.getSystemFunctions() );
}
catch (SQLException e) {
       System.out.println("SQLException caught in getSysFunctions: " + e.getMessage() );
}
finally {
        return coll;
}


}

// --------------------------------------------------------------------
public Collection getStrFunctions() {
Collection coll = null;
try {
    coll = parseFunctionsFromString( dbMetaData.getStringFunctions() ); 
}
catch (SQLException e) {
       System.out.println("SQLException caught in getStrFunctions: " + e.getMessage() );
}
finally {
        return coll;
}

}

// --------------------------------------------------------------------
public JPanel loadPanel( ArrayList info ) {

// create grid of row/column for the data 
JPanel panel = new JPanel( new GridLayout( 0, 2 ) );

for ( int n  = 0; n < info.size(); n++ ) {
     JTextField field = new JTextField( info.get( n ).toString() );
     if ( ( n % 2 ) == 0 ) {                    // even numbers are labels
        //field.setBackground( Color.lightGray ); 
	field.setBackground( new Color( 0x70E0E0 ) );
     }
     field.setEnabled( false );                // don't edit them 
     panel.add( field );
}

return panel;

}

// --------------------------------------------------------------------
public void output( String title, Collection outputCollection ) {

Iterator i = outputCollection.iterator();

System.out.println( "*********************************" );
System.out.println( "<<<< " + title + ">>>>" );
System.out.println( "*********************************" );
while ( i.hasNext() ) {
       System.out.println( i.next() + "\t" +   // the label 
                           i.next()  );  // the value 
}
System.out.println( "*********************************" );

}

}
