
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package chapter13;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.ResultSetMetaData;
import java.sql.ResultSet;
import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import chapter10.JDBCTypeMapper;

public class GeneralDAO  {

private Connection con;
private PreparedStatement preparedQuery; 
private ResultSet             resultSet;
private ResultSetMetaData     resultMD;

private GeneralAggregateVO     gvo = new GeneralAggregateVO();

// the default DataSource name 
private static String defaultDS = "movies";     

// -------------------------------------------------
public static void main( String args[] ) {

String     fileName = null;
String     query    = null;
GeneralDAO gd     = null;

try { 

     //
     // get a local object reference 
     //
     //gd = new GeneralDAO();
     gd = new GeneralDAO("knowledgebase");

     // 
     // if we didn't get a command line argument, just use the default file name 
     //
     if ( args.length == 1 ) 
        fileName = args[0];
     else 
        fileName = "query.sql";

    //
    // get the query 
    //
    query = gd.getQuery( fileName );
    if ( query == null ) {
        System.out.println( "No query found. Exiting ... " );
        System.exit( -1 );
     }

    //
    // process the query 
    //
    System.out.println("****************************");
    System.out.println("Processing file: " + fileName );
    System.out.println("Processing Query: " + query );
    System.out.println("****************************");

gd.gvo.clear();
gd.executeQuery( query ); 
gd.gvo.dumpContents();

     //
     // pass the connection into the processQuery method. 
     // it is responsible for executing the query 
     //
//     ResultSet rs = dq.processQuery( dq.con,  query );
   
     //
     // output results 
     //
 //    dq.outputResultSet( rs );
				
}
catch (Exception e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

    try {   
        // close the connection 
	if ( gd.con != null )
            gd.con.close(); 
    }
    catch (SQLException e) {
          e.printStackTrace();
    }
}

}
// ------------------------------------------------------------------


// *******************************************************************


public void executeQuery( String query ) throws SQLException {

// debug
if ( con == null ) 
    System.out.println("con is null.");


   //
   // execute the query and store in our local ResultSet
   //
   preparedQuery = con.prepareStatement( query ); 
   System.out.println("Preparing and executing query ... ");
   if ( preparedQuery != null ) {
       resultSet     = preparedQuery.executeQuery(); 
       resultMD      = resultSet.getMetaData();
   }
   else {
       System.out.println( "Null PreparedStatement. Quitting ... ");
       System.exit(-1);
   }


   // 
   // store the resultSet in our GeneralVO object 
   //
   System.out.println( "Loading results ... " );
   loadResults();

}

public void executeQuery() throws SQLException {

   // 
   // this asserts that executeQuery( string ) has been called previously
   //
   resultSet     = preparedQuery.executeQuery(); 
   resultMD      = resultSet.getMetaData();

   // store the resultSet in our GeneralVO object 
   System.out.println( "Loading results ... " );
   loadResults();
     
}

public GeneralDAO( String dataSourceName  ) {

try { 
     //
     // JNDI startup parameters are stored in the "jndi.properties" file in the 
     // classpath.
     //
     InitialContext ctx = new InitialContext( );

     //
     // get the DataSource from the JNDI name server 
     //
     DataSource ds = (DataSource) ctx.lookup( dataSourceName );

     //
     // get the connection from the DataSource
     //
     con = ds.getConnection( );
}
catch (NamingException e ) {
      System.out.println( "NamingException in GeneralDAO: " + e.getMessage() );
}
catch (SQLException e) {
      System.out.println( "SQLException in GeneralDAO: " + e.getMessage() );
}

}
// --------------------------------------------------------------

public GeneralDAO() {

     //
     // call constructor with the default DataSource name 
     //
     this( defaultDS );

}





// -------------------------------------------------------------------
//public ResultSet processQuery( String query ) {
//    return processQuery( con, query );
//}


public Connection getConnection() {

    return this.con;

}


// -------------------------------------------------------------------
//public ResultSet  processQuery( Connection con, String query ) {
//ResultSet rs = null;
//try {
//Statement stmt = con.createStatement();
//
//rs = stmt.executeQuery( query );
//
//}
//
//catch (SQLException e ) {
// display SQL specific exception information 
//     System.out.println("*************************" );
//     System.out.println("SQLException in processQuery: " + e.getMessage() );
//     System.out.println("** SQLState: " + e.getSQLState());
//     System.out.println("** SQL Error Code: " + e.getErrorCode());
//     System.out.println("*************************" );
//     e.printStackTrace();
//}
//finally {
// return rs;
//}
//}


// -------------------------------------------------------------------
//void outputResultSet( ResultSet rs ) {
//
//try { 
//    ResultSetMetaData resMetaData = rs.getMetaData();
//   while ( rs.next() ) {
//
//int colCount = resMetaData.getColumnCount();
// System.out.println( "**********************************" );
//       for (int n = 1; n <= colCount; n++ ) {
//             System.out.println( resMetaData.getColumnLabel( n )  + 
//                                    ": " + "\t\t" + 
//			             rs.getString( n ) );                     
//
//           }
//	    System.out.println( "**********************************" );
//
//    }
//}
//catch (SQLException e ) {
//    // display SQL specific exception information 
//   System.out.println("*************************" );
//  System.out.println("SQLException in outputResultSet: " + e.getMessage() );
// System.out.println("** SQLState: " + e.getSQLState());
//System.out.println("** SQL Error Code: " + e.getErrorCode());
//System.out.println("*************************" );
//e.printStackTrace();
//}
//}

// ------------------------------------------------------------------
// retrieves a query from a given filename 
public String getQuery( String fn ) {
String buffer  = null; // data buffer 
String retVal  = null; // query string to return 

try { 
     //
     // create a file reader for the file containing the query
     //
     BufferedReader reader = new BufferedReader ( new FileReader( fn ));

     // 
     // start reading the file 
     //
     buffer = reader.readLine();
     while ( buffer != null ) {

          if ( retVal == null ) {   // this is the first pass 
              retVal = buffer;
           }
          else {
              retVal += buffer;     // append 
          }
      //
      // continue reading the file 
      //
      buffer = reader.readLine();
     }

    //
    // close the file reader 
    //
    reader.close();
}
catch (FileNotFoundException e) {
     System.out.println("FileNotFoundException in getQuery: " + e.getMessage() );
}
catch (IOException e) { 
     System.out.println("IOException in getQuery: " + e.getMessage() );
}
finally { 
     return retVal;
}

}
// ---------------------------------------------------------------------

// returns the number of rows in a ResultSet 
public int getRows( ResultSet rs ) {
int rows = 0;
try { 

    while ( rs.next() ) {
            rows++;
    }

}
catch (SQLException e) {
      System.out.println("SQLException in getRows: " + e.getMessage() );
}
finally {
    return rows;
}

}


// set our resultset data to look like the gvo 
public void setAggregateVO( ) throws SQLException {

   for ( int n = 0; n < gvo.getRowCount(); n++ ) {

         // move to the row
         gvo.absolute( n );  

        // updates in this row 
       if ( gvo.getUpdateStatus( n ) ) { 
       
          // move to the row 
          resultSet.absolute( n + 1 );                          

          for ( int z = 0; z < gvo.getColumnCount(); z++ ) {  // set all columns 

		// update the object 
                resultSet.updateObject( (z + 1), gvo.getObject( z ) );

	   }  

       }     
   }

}

// load from resultset to aggregate vo 
public void loadAggregateVO( GeneralAggregateVO  gvo ) throws SQLException {

   this.gvo = gvo;
   setAggregateVO();   // set our resultset to look like the gvo 

}

public void deleteRow( int row ) throws SQLException {

    // delete our current row 
    resultSet.absolute( row + 1 );
    resultSet.deleteRow();

}
// ------------------------------------------------------------------------------

public void applyUpdates ( GeneralAggregateVO gvo ) throws SQLException {
// 
// apply the gvo before applying the updates, to get the two in synch
// this is needed for RMI access of the gdao 
//

try {

   // 
   // load the general aggregate value object passed in 
   //
   this.loadAggregateVO( gvo );

   // 
   // apply updates 
   //
   this.applyUpdates(); 
}
catch (Exception e)  {
      System.out.println( "Exception in applyUpdates: " + e.getMessage() );
}

}





// apply updates from the gvo
// this currently doesn't require setAggregateVO to be called 
// since it calls updateObject for each updated row/column combination 
// 
public void applyUpdates() throws SQLException {

   for ( int n = 0; n < gvo.getRowCount(); n++ ) {

          // move to the row
	  gvo.absolute( n );            // our aggregate VO data 
          resultSet.absolute( n + 1 );  // the underlying resultset 

         // if this is an insert update, we need to move to an 'insert' row 
	 if ( gvo.getInsertStatus( n ) ) {   // is this an insert row 
	     resultSet.moveToInsertRow();
	 }

       if ( gvo.getUpdateStatus( n ) ) { // any update in this row ?

           // update only those cols that need updating
	   for ( int z = 0; z < gvo.getColumnCount(); z++ ) { 
	       if ( gvo.getUpdateStatus( n, z ) ) {   // update status for the row, column
	           resultSet.updateObject( (z + 1), gvo.getObject( z ));
               }
	   }

	 if ( gvo.getInsertStatus( n ) ) {   // this an insert row 
	     resultSet.insertRow();          // insert the row
	 }
         else {                              // this is an update 
	  resultSet.updateRow();             // update the row 
	 }
	}
   }

}

// ----------------------------------------------------------------------------
public void loadResults() throws SQLException {

      // by default we use the internal resultset  
      loadResults( resultSet ); 
}

// ----------------------------------------------------------------------------
// load results from the resultset into the GeneralAggregateVO
public void loadResults( ResultSet rs ) throws SQLException {

      // clear GeneralAggregateVO 
      gvo.clear();

     // set the columns in the gvo
     for ( int c = 1; c <= resultMD.getColumnCount(); c++) { 

          // ** try getColumnLabel ** 
          gvo.setColKey( resultMD.getColumnName( c ), (c-1) );
     }

     // for each row 
     while ( rs.next() ) {

       for ( int n = 1; n <= resultMD.getColumnCount(); n++ )  {
       
	    // apply some general conversion 
	    // gvo.addObject( JDBCTypeMapper.getColumnDataString( n, rs ) );
	    gvo.addObject( JDBCTypeMapper.getColumnData( n, rs ) );
	    // gvo.addObject( rs.getObject( n ) );
       }

         // add the row to our aggregate object 
         gvo.addRow();

     }


}    //  
// ----------------------------------------------------------------------------

public void clearUpdates() {

    // reset update flags
    // does NOT reload data to pre-updates state
    gvo.clearUpdates();
}



// ----------------------------------------------------------------------------

public GeneralAggregateVO getGeneralAggregateVO() {

        return gvo;
}

}
