
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */

package chapter14;

import java.sql.SQLException;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.NotBoundException;
import java.net.MalformedURLException;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.event.*;
import java.awt.*;

import db.GeneralAggregateVO;

// client for RMI component which encapsulates data access 
public class RemoteTableBrowser {

dataTableModel dtModel=null;
JTable table;

GeneralAggregateVO gvo;
RemoteDAO      gdo;
String         query = "select * from movies";

JFrame    frame;
JButton   enterQuery; 
JButton   executeQuery; 
JButton   cancelUpdates;
JButton   executeUpdate;
JButton   executeInsert; 
JButton   executeDelete; 
JButton   exitButton; 
JButton   deleteButton;
JPanel    buttonPanel;
JPanel    contentPanel;
JTextArea queryText;

public static void main( String args[] ) {

      RemoteTableBrowser remoteTB = new RemoteTableBrowser();

      remoteTB.getData();

      if ( remoteTB.gvo != null ) // we have a gvo so can build gui 
          remoteTB.buildGUI();


} // end main

// -------------------------------------------------------------

protected void getData() {

      try {
      //
      // create general Data Object for dynamic queries 
      //
      //gdo = new GeneralDAO();
      
          // do a remote lookup
	  System.out.println("Getting RemoteDAO ... " );
          gdo = ( RemoteDAO ) Naming.lookup( "remote_general_dao" );

          // execute the query and store results 
	  System.out.println("Executing Query ... " );
          gdo.executeQuery( query );
      
          // get the GeneralAggregateVO - aggregate Value Object with data and access methods
	  System.out.println("Retrieving gvo ... " );
          gvo = gdo.getGeneralAggregateVO();

       }
       catch (SQLException e) {

              System.out.println("SQLException in TableExample1. Message: " + e.getMessage());
	      e.printStackTrace();
	      JOptionPane.showMessageDialog( null,
	                                     "Database error on load. Message: " + e.getMessage(),
	                                     "Error",
			                     JOptionPane.ERROR_MESSAGE);
       }
       catch (RemoteException ex) {
              System.out.println("RemoteException in RemoteTableBrowser. Message: " + ex.getMessage());
	      ex.printStackTrace();
       }
       catch (NotBoundException ex) {
              System.out.println("NameNotBoundException in RemoteTableBrowser. Message: " + ex.getMessage());
	      ex.printStackTrace();
       }
       catch (MalformedURLException ex) {
              System.out.println("MalformedURLException in RemoteTableBrowser. Message: " + ex.getMessage());
	      ex.printStackTrace();
       }
       finally {

      // print message about rows loaded 
       if ( gvo != null ) {
           JOptionPane.showMessageDialog( null,
	                                "Loaded " + gvo.getRowCount() + " rows.",
	                                "Message",
	    		                 JOptionPane.INFORMATION_MESSAGE);
       }
     }

}


protected void buildGUI() {

      //
      // create the frame and content pane
      //
      frame = new JFrame("Database Table Browser");
      contentPanel = new JPanel(new BorderLayout());
      frame.setContentPane(contentPanel);

      //
      // load our table model with the data using our object instance 
      //
      dtModel = this.new dataTableModel(  );

      // create GUI table 
      table = new JTable( dtModel );

      //
      // add the table to the scrollpane
      //
      JScrollPane scrollpane = new JScrollPane( table );
      scrollpane.setPreferredSize( new Dimension( 700, 300 ) );

      //
      // set the window closing event
      //
      frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
      });

      // 
      // text area for query input
      //
      queryText = new JTextArea( query, 5, 80 );
      queryText.setLineWrap( true );
      queryText.setWrapStyleWord( true );

      //
      // create a scrollpane for the text area
      //
      JScrollPane queryPane = new JScrollPane( queryText );
      queryPane.setVerticalScrollBarPolicy(
                                 JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
      queryPane.setPreferredSize(new Dimension(250, 250));

      // create buttons and handlers 
      createButtonsHandlers();

      //
      // add all constituent panels and panes to the contentPanel
      //
      contentPanel.add( scrollpane, BorderLayout.NORTH ); // data output
      contentPanel.add( queryPane, BorderLayout.CENTER ); // query
      contentPanel.add( buttonPanel, BorderLayout.SOUTH ); // buttons

      //
      // now let's roll ... 
      //
      frame.pack();
      frame.setVisible(true);

}

// -------------------------------------------------------------
protected void createButtonsHandlers() {

      //
      // create some buttons and corresponding listeners
      //
      enterQuery    = new JButton ( "Enter Query" );
      executeQuery  = new JButton ( "Execute Query" );
      cancelUpdates = new JButton ( "Cancel Updates" );
      executeUpdate = new JButton ( "Apply" );
      executeInsert = new JButton ( "Insert Row" );
      executeDelete = new JButton ( "Delete Row" );
      exitButton    = new JButton ( "Exit" );

     // button listener/handlers 
     exitButton.addActionListener( new ActionListener() {
           public void actionPerformed( ActionEvent e) { System.exit(1); }
      } );
     // -------------------------------------------------------

     // button listener/handlers 
     executeInsert.addActionListener( new ActionListener() {
           public void actionPerformed( ActionEvent e) { 
 
              gvo.appendRow(); // append a new row 
              dtModel.fireTableStructureChanged();

	   }
      } );

     // -------------------------------------------------------

     // button listener/handlers 
     executeDelete.addActionListener( new ActionListener() {
           public void actionPerformed( ActionEvent e) { 

              // go directly to the database and delete the row 
	      // then execute the query again 

              // which row to delete 
	      int confirm  = 0;
              int row = table.getSelectedRow();
	      if ( row == -1 ) {
	            JOptionPane.showMessageDialog( null,
	                                           "No row selected for delete.",
	                                           "Error",
			                           JOptionPane.ERROR_MESSAGE);
	      }
	      else {                 // we have a row to delete. get confirmation 
                    confirm = JOptionPane.showConfirmDialog( null, 
		                                             "Row " + row + " selected for delete ?"); 
	      }
 
              if ( confirm == JOptionPane.YES_OPTION ) {
	         try {
                     gdo.deleteRow( row ); 
                     gdo.executeQuery( query );
                     gvo = gdo.getGeneralAggregateVO();

		     // if at this point, success 
	             JOptionPane.showMessageDialog( null,
	                                           "Row " + row + " has been deleted.", 
	                                           "Message",
			                           JOptionPane.INFORMATION_MESSAGE);
		  }
		  catch (SQLException ex) {
                    System.out.println( "SQLException on query execution. Message: " + ex.getMessage() );
	            JOptionPane.showMessageDialog( null,
	                                           "Error on query execution. Message: " + ex.getMessage(), 
	                                           "Error",
			                           JOptionPane.ERROR_MESSAGE);
		  }
		  catch (RemoteException remoteExc) {
                    System.out.println( "RemoteException on query execution. Message: " + remoteExc.getMessage() );
                    System.out.println( "RemoteException on query execution. toString: " + remoteExc );
		  }
		  catch (Exception exc) {
                    System.out.println( "Exception on query execution. Message: " + exc.getMessage() );
                    System.out.println( "Exception on query execution. toString: " + exc );
		    
		  }
              }
	      else {               // user chose not to delete 

	            JOptionPane.showMessageDialog( null,
	                                           "No rows deleted.",
	                                           "Message",
			                           JOptionPane.INFORMATION_MESSAGE);
	      }

              // this will redraw the table
              dtModel.fireTableStructureChanged();

	   }
      } );


     // -------------------------------------------------------
     executeQuery.addActionListener( new ActionListener() {
           public void actionPerformed( ActionEvent e) {
                 // use DynamicQuery object to execute the query
                 // and process the results
               try {
                    gdo.executeQuery( queryText.getText() );             
	            JOptionPane.showMessageDialog( null,
	                                           "Loaded " + gvo.getRowCount() + " rows.",
	                                           "Message",
			                           JOptionPane.INFORMATION_MESSAGE);
                    // get the new GeneralAggregateVO 
                    gvo = gdo.getGeneralAggregateVO();
                    dtModel.fireTableStructureChanged();
                    dtModel.fireTableDataChanged();
               }
	       catch (SQLException ex) {
                    System.out.println( "SQLException on query execution. Message: " + ex.getMessage() );
	            JOptionPane.showMessageDialog( null,
	                                           "Error on query execution. Message: " + ex.getMessage(), 
	                                           "Error",
			                           JOptionPane.ERROR_MESSAGE);
	       }
	       catch (Exception exc) {
                    System.out.println( "Exception on query execution. Message: " + exc.getMessage() );
	       }
	  }
      } );

      enterQuery.addActionListener( new ActionListener() {
          public void actionPerformed( ActionEvent e) {
                 queryText.setText( "" );
          }
      } );

      executeUpdate.addActionListener( new ActionListener() {
           public void actionPerformed( ActionEvent e) {
                 // 
                 // execute all pending updates 
		 //
	     try { 
                gdo.applyUpdates( gvo );    // must pass the gvo to RMI server          
	        JOptionPane.showMessageDialog( null,
		                               "Updates applied to database.",
	                                       "Message",
			                        JOptionPane.INFORMATION_MESSAGE);
	     }
	     catch (SQLException ex) {
                    System.out.println( "SQLException on query execution. Message: " + ex.getMessage() );
	            JOptionPane.showMessageDialog( null,
	                                           "Error on query execution. Message: " + ex.getMessage(), 
	                                           "Error",
			                           JOptionPane.ERROR_MESSAGE);
	     }
	     catch (RemoteException remoteExc) {
                    System.out.println( "RemoteException on query execution. Message: " + remoteExc.getMessage() );
                    System.out.println( "RemoteException on query execution. toString: " + remoteExc );
	     }
	     catch (Exception exc) {
                    System.out.println( "Exception on query execution (executeUpdate handler). Message: " + exc.getMessage() );
		    exc.printStackTrace();
	     }
	   }
      } );

      cancelUpdates.addActionListener( new ActionListener() {
          public void actionPerformed( ActionEvent e) {
                 gvo.clearUpdates();
	         JOptionPane.showMessageDialog( null,
		                               "Updates were cleared.",
	                                       "Message",
			                        JOptionPane.INFORMATION_MESSAGE);
          }
      } );

      //
      // create a panel for the buttons and add them
      //
      buttonPanel = new JPanel();
      buttonPanel.add( executeQuery );
      buttonPanel.add( cancelUpdates );
      buttonPanel.add( executeUpdate );
      buttonPanel.add( enterQuery );
      buttonPanel.add( exitButton );
      buttonPanel.add( executeInsert );
      buttonPanel.add( executeDelete );

}




// --------------------------------------------------------------
class dataTableModel extends AbstractTableModel {

boolean updateFlag = false;  // set true if we have pending updates

   public dataTableModel( ) {
   }
// ------------------------------------------------

   public int getColumnCount() {
       return gvo.getColumnCount();
   }
// ------------------------------------------------

   public int getRowCount() {
       return gvo.getRowCount();
  }
// ------------------------------------------------
   public Object getValueAt(int row, int col) {
         gvo.absolute( row );
         return  gvo.getObject( col );
  }

  // ------------------------------------------------
  public String getColumnName( int col ) {
      return gvo.getColumnName( col );
  }

// ------------------------------------------------

 // ** need to add this to the gvo 
  public boolean isCellEditable( int row, int col ) {
      return true;
}

// ------------------------------------------------
  public Class getColumnClass( int col ) {
   //return getValueAt( 0, col ).getClass();
   return new Object().getClass();
  }

// ------------------------------------------------
  // called when data has changed for a column
  public void setValueAt(Object value, int row, int col) {

          // set the update flag 
          updateFlag = true;

          // update the gvo
          gvo.absolute( row );
          gvo.updateObject( col, value );

	  fireTableCellUpdated( row, col ); // update just this row

   }

// --------------------------------------------------------

} // -- end inner class



} // -- end class
