
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package chapter15;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.ResultSet;
import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import java.util.Vector;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

public class ObjectExample1 {


public static void main( String args[] ) {

Connection con=null;
ResultSet rs = null;

try { 


    ObjectExample1 objectExample = new ObjectExample1();

    // 
    // obtain a connection 
    //
    con = objectExample.getConnected();


    //
    // delete any existing record 
    //
    Statement stmt = con.createStatement();
    stmt.executeQuery( "delete  from objects where obj_name = 'customer_purchases-4'" );
		    
    //
    // create a prepared statement 
    //
    PreparedStatement pstmt = con.prepareStatement( "insert into objects values ( ?, ? )" );

    //
    // set the 'name' of the object
    //
    pstmt.setString( 1, "customer_purchases-4" ); // describes object

    //
    // gather customer purchases in a collection 
    //
    Vector purchases = objectExample.getCustomerPurchases();

    //
    // Output the collection to an internal bytestream
    //
    ByteArrayOutputStream byteArrayStream = new ByteArrayOutputStream();
    ObjectOutputStream    out             = new ObjectOutputStream( byteArrayStream );
    out.writeObject( (Object) purchases );

    //
    // Create an InputStream on the bytearray (created from the byte array stream)
    // set the Input for the PreparedStatement to the ByteArrayInputStream
    //
    ByteArrayInputStream input = new ByteArrayInputStream( byteArrayStream.toByteArray() );
    pstmt.setBinaryStream( 2, input, 0 ); 

    //
    // update the database 
    //
    int recCount = pstmt.executeUpdate();

    //
    // read the data
    //
    stmt = con.createStatement();

    //
    // retrieve the object we inserted 
    //
    rs = stmt.executeQuery( "select * from objects where obj_name like 'customer_purchases-4'" );

    //
    // get the record and output the object 
    //
    while ( rs.next() ) {

       ObjectInputStream objInput = new ObjectInputStream( rs.getBinaryStream( "obj_ref" ));

       System.out.println( "Found object. Contents: " );

       //
       // records were stored in Vector objects
       //
       Vector retrievedPurchases = (Vector) objInput.readObject();  
       for (int n = 0; n < retrievedPurchases.size(); n++ ) {
           

	   // 
	   // cast each element of the Vector to our object type 
	   //
           CustomerPurchase custPurch = (CustomerPurchase) retrievedPurchases.get( n );

	   System.out.println("***************************" );
           System.out.println("title: " + custPurch.getTitle() );
           System.out.println("stockNumber: " + custPurch.getStockNumber() );
           System.out.println("Date: " + custPurch.getDate() );
           System.out.println("Amount: " + custPurch.getAmount() );
           
       }
    }
				
}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in main: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
 }
 catch (Exception e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
 }
 finally {

    try {   
        // close the connection 
	if ( con != null )
            con.close(); 
	System.exit(1); 
    }
    catch (SQLException e) {
          e.printStackTrace();
    }
 }
}
// --------------------------------------------------------------------

public Connection getConnected() {

Connection con = null;

try { 

     // 
     // get the InitialContext
     //
     InitialContext ctx = new InitialContext(  );

     //
     // obtain a DataSource to our 'test' database
     //
     DataSource  ds   = (DataSource) ctx.lookup("test");

     //
     // create a logwriter 
     //
     PrintWriter pw  = ds.getLogWriter();
     if (pw == null) 
         ds.setLogWriter(new PrintWriter( new FileWriter("test.log")) );

    pw  = ds.getLogWriter();

    //
    // write to the log for this DataSource 
    //
    if ( pw != null ) 
        pw.println("Program ObjectExample1 has a connection.  ");
    pw.flush();

   //
   // get a connection
   //
   con = ds.getConnection();

}
catch (SQLException e) {
   System.out.println( "SQLException caught in getConnected: " + e.getMessage() );
    e.printStackTrace();
}
catch (NamingException e) {
   System.out.println( "NamingException caught in getConnected: " + e.getMessage() );
}
catch (IOException e) {
   System.out.println( "IOException caught in getConnected: " + e.getMessage() );
}
finally {

    //
    // return the connection 
    //
    return con;
}


}

// --------------------------------------------------------------------
public Vector getCustomerPurchases() {

//
// create a new CustomerPurchase object
//
CustomerPurchase custPurch = new CustomerPurchase();

//
// create a new Vector 
//
Vector purchases = new Vector();

//
// set the object members
//
custPurch.setTitle( "The Evil Null Reference" );
custPurch.setDate( "01/02/2001" );
custPurch.setAmount( 29.22 );
custPurch.setStockNumber( "X334334" );

// 
// add to the Vector 
//
purchases.add( custPurch );

//
// add a new object 
//
custPurch = new CustomerPurchase();
custPurch.setTitle( "Another Lonely One" );
custPurch.setDate( "01/02/2002" );
custPurch.setAmount( 59.22 );
custPurch.setStockNumber( "X333484" );

purchases.add( custPurch );

//
// add a new object 
//
custPurch = new CustomerPurchase();
custPurch.setTitle( "The Last Compile" );
custPurch.setDate( "01/02/2003" );
custPurch.setAmount( 59.22 );
custPurch.setStockNumber( "X333434" );

purchases.add( custPurch );

//
// add a series of objects
//
for ( int n = 0;n < 10;n++) {
     custPurch = new CustomerPurchase();
     custPurch.setTitle( "The Last Compile: The Sequel: " + n);
     custPurch.setDate( "01/02/2003" );
     custPurch.setAmount( 109.22 );
     custPurch.setStockNumber( "X32113434" );
     purchases.add( custPurch );
}
    

//
// return the collection (Vector) of objects 
//
return purchases;

}


}
