/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */

package chapter16;

//import javax.servlet.*;
import java.util.Vector;
import java.util.Collection;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;

import db.DBUtil;


public class CategoriesDAO {

// private members
private String category;
private String description;

// hold query strings 
private String insertStmtStr;
private String updateStmtStr;
private String deleteStmtStr;
private String selectStmtStr;
private String selectCategoryListStr;

// hold prepared statements 
private PreparedStatement insertStmt;
private PreparedStatement updateStmt;
private PreparedStatement deleteStmt;
private PreparedStatement selectStmt;
private PreparedStatement selectCategoryListStmt;

DBUtil dbutil;

// getXXXX methods 
public String getCategory() {
   return this.category;
}

public String getDescription() {
    return this.description;
}

// setXXXX methods 
public void setCategory( String category ) {
   this.category = category;
}

public void setDescription( String description ) {
   this.description = description;
}

// convenience methods 
public void createPreparedStatements() {

try {

insertStmtStr = "insert into categories( category, description) values (?,?)";

insertStmt = dbutil.createPreparedStatement( insertStmtStr ); 
   
updateStmtStr = "update categories set description = ? where category = ? ";

updateStmt = dbutil.createPreparedStatement( updateStmtStr );

deleteStmtStr = "delete from categories where category = ?"; 

deleteStmt = dbutil.createPreparedStatement( deleteStmtStr ); 

selectStmtStr = "select category, description from categories where category = ?";

selectStmt = dbutil.createPreparedStatement( selectStmtStr ); 

selectCategoryListStr = "select category, description from categories";
selectCategoryListStmt = dbutil.createPreparedStatement( selectCategoryListStr ); 

}
catch( Exception e) {

  System.out.println("Exception in categoriesDAO.CreatePreparedStatement(): " + e.getMessage() );

}

}

public void updateDAO() throws SQLException {

    updateStmt.setString(1, getDescription() ); 
    updateStmt.setString(2, getCategory() ); 

    updateStmt.executeUpdate();

}

public void insertDAO() throws SQLException {

     insertStmt.setString(1, getCategory() ); 
     insertStmt.setString(2, getDescription() ); 

     insertStmt.executeUpdate();
}

public void deleteDAO() throws SQLException {

      deleteStmt.setString(1, getCategory() ); 

      deleteStmt.executeUpdate();

}

public void loadDAO( String category )  throws SQLException {

 selectStmt.setString(1, category ); 

 ResultSet rs = selectStmt.executeQuery(); 

if ( rs.next() ) { 
 setCategory( rs.getString(1) );
 setDescription( rs.getString(2) ); 
}

}

// load values from ValueObject
public void loadDAO( CategoriesVO vo ) {

setCategory( vo.getCategory() ); 
setDescription( vo.getDescription() ); 

}

// set ValueObject with internal member values 
public void setVO( CategoriesVO vo ) {

vo.setCategory( this.getCategory() );
vo.setDescription( this.getDescription() ); 

}

public Collection getCategoryList() throws SQLException {
Vector v = new Vector();
CategoriesVO vo = null;

ResultSet rs = selectCategoryListStmt.executeQuery();

while ( rs.next() ) { 
      vo = new CategoriesVO(); 
      vo.setCategory( rs.getString(1) );
      vo.setDescription( rs.getString(2) ); 
      v.add( vo );
}

return v; 

}

// constructor 
public CategoriesDAO() {

     // create db wrapper
     dbutil = new DBUtil();

     // create SQL prepared statements
     createPreparedStatements();
     

}



}
