
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */

package chapter16;

import java.util.Vector;
import java.util.Iterator;
import java.util.Collection;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;

import db.DBUtil;

public class Knowledge_baseDAO {

// private members
private int doc_key;
private String doc_name;
private String doc_location;
private String post_user;
private int link_doc;
private int level;
private String entry_date; // date 
private String date_submitted; // date 
private String category;
private int base_doc_key;


// wraps JDBC methods 
DBUtil           dbutil;
DatabaseMetaData dbMetaData;


// string to hold SQL statements 
private String insertStmtStr;
private String updateStmtStr;
private String deleteStmtStr;
private String selectStmtStr;
private String selectAllStmtStr;
private String selectFilterStmtStr;


// Prepared SQL statements
PreparedStatement insertStmt;
PreparedStatement updateStmt;
PreparedStatement deleteStmt;
PreparedStatement selectStmt;
PreparedStatement selectAllStmt;
PreparedStatement selectFilterStmt;

// getXXXX methods
public int getDoc_key() {

  return doc_key;

}

public String getDoc_name(){ 

  return doc_name; 

}


public int getBase_doc_key() {
   
   return this.base_doc_key;

}

public int getLink_doc() {

   return this.link_doc;

}

public String getDoc_location() {

  return doc_location;

}

public String getPost_user() { 

  return post_user; 
}
public int getLevel() { 

   return level;

}

public String getEntry_date() { 

   return entry_date; 

}

public String getDate_submitted() {

   return this.date_submitted; 
}

public String getCategory() { 

   return category;

}


// setXXXX methods 

public void setDoc_name( String doc_name ) {

    this.doc_name = doc_name; 

}

public void setDoc_key( int doc_key ) { 
     
    this.doc_key = doc_key;

}

public void setDoc_location( String doc_location ) {
 
    this.doc_location = doc_location;

}

public void setPost_user( String post_user ) {

     this.post_user = post_user; 
}

public void setLevel( int level ) {

    this.level = level;

}

public void setLink_doc( int link_doc ) {

   this.link_doc = link_doc;

}

public void setBase_doc_key( int base_doc_key ) {

    this.base_doc_key = base_doc_key;

}

public void setEntry_date( String entry_date ) {

    this.entry_date = entry_date;
}


public void setDate_submitted( String date_submitted ) {

   this.date_submitted = date_submitted; 

}

public void setCategory( String category ) {

    this.category = category;

}


// convenience methods 

public void createPreparedStatements( ) throws Exception {

// ** maybe move the initialization of these strings to instance members ?? ** 


try {

// ** 
// add date_submitted + entry_date 
    insertStmtStr = "insert into knowledge_base " + 
                    " (doc_key,doc_name,category,post_user,doc_location,link_doc, level, base_doc_key, entry_date, date_submitted) values " +  
                    " ( ?,    ?,       ?,       ?,        ?,           ?,        ?,     ?,            ?,          ? ) "  ;

    insertStmt = dbutil.createPreparedStatement( insertStmtStr ); 


    updateStmtStr = "update knowledge_base " +  
                 " set doc_name = ?, " +
        	 "  doc_location = ?, " +
        	 "  category = ?, "  +
        	 "  post_user = ?, "  +
          	 "  level = ?, "  +
          	 "  link_doc = ?, "  +
          	 "  entry_date = ? "  +
                 "  where doc_key = ? "  ;

    updateStmt = dbutil.createPreparedStatement( updateStmtStr ); 

    deleteStmtStr = "delete from knowledge_base " +
	            "       where doc_key = ?" ;

    deleteStmt = dbutil.createPreparedStatement( deleteStmtStr ); 


    selectStmtStr = "select doc_key,doc_name,category,post_user,doc_location,link_doc, " +  
                    " base_doc_key,entry_date, date_submitted,level " +  
                    " from knowledge_base " + 
                    " where doc_key = ? ";

    selectStmt = dbutil.createPreparedStatement( selectStmtStr ); 

    selectAllStmtStr = "select doc_key,doc_name,category,post_user,doc_location,link_doc, " +  
                    " base_doc_key,entry_date, date_submitted,level " +  
                    " from knowledge_base " + 
                    " order by base_doc_key, doc_key, date_submitted, entry_date";

    selectAllStmt = dbutil.createPreparedStatement( selectAllStmtStr );

    String selectFilter1 = " select " + 
    " doc_key, doc_name,category, post_user, doc_location, link_doc, " + 
    " base_doc_key,entry_date, date_submitted, level " + 
    " from knowledge_base " + 
    " where base_doc_key in (" + 
    " select knowledge_base.doc_key " + 
    " from knowledge_base, base_keys  " + 
    " where keyword in (?,?,?,?) and "  + 
    " knowledge_base.doc_key = base_keys.doc_key ) " + 
    " union "  + 
    " select "  + 
    " knowledge_base.doc_key, doc_name,category, post_user, doc_location, link_doc, " + 
    " base_doc_key,entry_date, date_submitted, level " + 
    " from knowledge_base, base_keys  " + 
    " where keyword in (?,?,?,?) and " + 
    " knowledge_base.doc_key = base_keys.doc_key " + 
    " order by knowledge_base.doc_key, base_doc_key, date_submitted, entry_date ";


     // some dbs don't support 'union'
     String selectFilter2 = "select   knowledge_base.doc_key, doc_name,category, post_user, doc_location, link_doc,   " + 
      " base_doc_key,entry_date, date_submitted, level   " + 
      " from knowledge_base, base_keys   " + 
      " where keyword in ( ?,?,?,?) and "  + 
      " knowledge_base.doc_key = base_keys.doc_key " + 
      " order by knowledge_base.doc_key, base_doc_key, date_submitted, entry_date ";


  //
  // if we are using MySQL, we will not use the 'union' clause 
  //
  if ( dbMetaData.getDatabaseProductName().equals( "MySQL")  ) {
      selectFilterStmtStr = selectFilter2;
  }
  else {
      selectFilterStmtStr = selectFilter1;
  }

   selectFilterStmt = dbutil.createPreparedStatement( selectFilterStmtStr ); 

}
    
catch (SQLException e) {

   throw new Exception("SQLException thrown in createPreparedStatements(): " + e.getMessage() );

} 
catch (Exception e)  {

   throw new Exception("Exception thrown in createPreparedStatements(): " + e.getMessage() );

}

}


public int insertDAO( ) throws SQLException, Exception {

int doc_key = 0;

try {
  //
  // get the doc_key from a sequence or from auto_increment 
  // if this is MySQL and we're using an autoincrement column we will get the doc_key at the end 
  //
  if ( dbMetaData.getDatabaseProductName().equals( "MySQL")  ) {
       doc_key = 0;                 // 0 means column value will be set automatically
  }
  else {
       //
       // otherwise we will use a 'sequence' to get the doc_key 
       //
       doc_key = generateDoc_key();
  }
  
  //
  // set elements 
  //
  insertStmt.setInt(1, doc_key );                 
  insertStmt.setString(2, getDoc_name() ); 
  insertStmt.setString(3, getCategory() ); 
  insertStmt.setString(4, getPost_user() ); 
  insertStmt.setString(5, getDoc_location() ); 
  insertStmt.setInt(6, getLink_doc() ); 
  insertStmt.setInt(7, getLevel() );
  
  insertStmt.setString(9, getEntry_date() );
  insertStmt.setString(10, getDate_submitted() );
  //
  // ** force base_doc_key to be the doc_key for the base document ** 
  // ** we also need to know the doc key for our document 
  // 
  if (getBase_doc_key() == 0 ) // this is the base document 
     insertStmt.setInt(8, doc_key );
  else                         // not the base document, so set the base_doc_key to that of the base document
     insertStmt.setInt(8, getBase_doc_key() ); 
  
  int retval = insertStmt.executeUpdate(); 
  
  
  if ( doc_key == 0 ) { // need to get the real doc_key 
  
  // ** this is a hack because mm.mysql jdbc driver doesn't return the auto-increment value right now 
  //
  ResultSet rs = dbutil.executeDBQuery("select max(doc_key) from knowledge_base");
  if ( rs.next() ) {
       doc_key = rs.getInt(1);
  }
  
  //
  // need to go back and update the doc_key if this is the base_document (first in thread) 
  //
  if (getBase_doc_key() == 0 ) {        // this is the base document 
      dbutil.executeUpdDBQuery( "update knowledge_base set base_doc_key = " + doc_key + 
                            " where doc_key = " + doc_key ); 
  }
  
  }
 }
 catch (SQLException e) {
        System.out.println("SQLException caught in insertDAO: " + e.getMessage() );
	e.printStackTrace();
 }
 catch (Exception e) { 
        System.out.println("SQLException caught in insertDAO: " + e.getMessage() );
	e.printStackTrace();
 }
 finally {
    return doc_key;
 }

}

// ---------------------------------------------------------------------------------------
public void updateDAO( ) throws SQLException {

// set elements 
updateStmt.setString(1, getDoc_name() );
updateStmt.setString(2, getDoc_location() ); 
updateStmt.setString(3, getCategory() ); 
updateStmt.setString(4, getPost_user() ); 
updateStmt.setInt(5, getLevel() ); 
updateStmt.setInt(6, getLink_doc() ); 
updateStmt.setString(7, getEntry_date() ); 
updateStmt.setInt(8, getDoc_key() ); 

int retval = updateStmt.executeUpdate(); 

}

public void deleteDAO( ) throws SQLException {

// set elements
deleteStmt.setInt(1, getDoc_key() );

int retval = deleteStmt.executeUpdate(); 

}

public void loadDAO( int doc_key ) throws Exception {

try {

selectStmt.setInt(1, doc_key ); 

ResultSet rs = selectStmt.executeQuery(); 

if ( rs.next() )  {

    setDoc_key( rs.getInt(1) );
    setDoc_name( rs.getString(2) );
    setCategory( rs.getString(3) );
    setPost_user( rs.getString(4) );
    setDoc_location( rs.getString(5) );
    setLink_doc( rs.getInt(6) ); 
    setBase_doc_key( rs.getInt(7) ); 
    setEntry_date( rs.getString(8) ); 
    setDate_submitted( rs.getString(9) ); 
    setLevel( rs.getInt(10) ); 

}

}
catch (SQLException e) {
   System.out.println("SQLException thrown in Knowlege_baseDAO.loadDAO(): " + e.getMessage() ); 
   throw new Exception( "Exception in Knowlege_baseDAO.loadDAO(): " + e.getMessage() ); 
}

}

// set DAO members from ValueObject
public void loadDAO( Knowledge_baseVO knowledge_base ) {

setDoc_key( knowledge_base.getDoc_key() );
setDoc_name( knowledge_base.getDoc_name() );
setCategory( knowledge_base.getCategory() );
setPost_user( knowledge_base.getPost_user() );
setDoc_location( knowledge_base.getDoc_location() );
setLink_doc( knowledge_base.getLink_doc() ); 
setBase_doc_key( knowledge_base.getBase_doc_key() ); 
setLevel( knowledge_base.getLevel() ); 
setEntry_date( knowledge_base.getEntry_date().toString() );
setDate_submitted( knowledge_base.getDate_submitted().toString() );

}

// set ValueObject members from DAO
public void setVO( Knowledge_baseVO vo ) throws Exception {

vo.setDoc_key( this.getDoc_key() );
vo.setDoc_name( this.getDoc_name() );
vo.setCategory( this.getCategory() );
vo.setPost_user( this.getPost_user() );
vo.setDoc_location( this.getDoc_location() );
vo.setLink_doc( this.getLink_doc() ); 
vo.setLevel( this.getLevel() );
vo.setBase_doc_key( this.getBase_doc_key() );
vo.setEntry_date( this.getEntry_date() );
vo.setDate_submitted( this.getDate_submitted() ); 

}

// retrieve a collection of all knowledge_base record ValueObjects 
Iterator getAll() throws Exception {
Vector v = new Vector();

try {

ResultSet rs = selectAllStmt.executeQuery();

while ( rs.next() ) {

    Knowledge_baseVO vo = new Knowledge_baseVO();

    vo.setDoc_key( rs.getInt(1) );
    vo.setDoc_name( rs.getString(2) );
    vo.setCategory( rs.getString(3) );
    vo.setPost_user( rs.getString(4) );
    vo.setDoc_location( rs.getString(5) );
    vo.setLink_doc( rs.getInt(6) ); 
    vo.setBase_doc_key( rs.getInt(7) ); 
    vo.setEntry_date( rs.getString(8) ); 
    vo.setDate_submitted( rs.getString(9) ); 
    vo.setLevel( rs.getInt(10) ); 

    v.add( vo ) ; 

 }


}
catch (SQLException e) {
   throw new Exception("SQLException caught in knowledge_baseDAO.getAll(): " + e.getMessage() ); 
}

// return the iterator for this collection
return v.iterator();

}

private int generateDoc_key() throws Exception {

try {

ResultSet rs = dbutil.executeDBQuery( "select nextval('doc_key')" );
if ( rs.next() )
   return rs.getInt(1);
else 
   return 0; 
}
catch (SQLException e) {

   throw new Exception( "Error in doc_key generation: " + e ); 

}

}
// ----- 

Iterator getFilteredList( Collection criteria ) throws Exception {
Vector v = new Vector();

try {

Iterator i = criteria.iterator();
int n = 1;

// currently, only store four search keywords

//
// if using MySQL, we don't have a 'union' clause 
//
if ( dbMetaData.getDatabaseProductName().equals( "MySQL" ) ) {
    for ( n = 1; n < 5 && i.hasNext(); n++ )  {
         selectFilterStmt.setString( n, (String) i.next() );
         }
}
else { 
     //
     // using a Union statement and the search is repeated, we need to set these keywords twice
     //
     for ( n = 1; n < 5 && i.hasNext(); n++ )  {
          selectFilterStmt.setString( n, (String) i.next() );
     }

     //
     // set the next 4
     //
     i = criteria.iterator();
     for ( n = 1; n < 5 && i.hasNext(); n++ )  {
          selectFilterStmt.setString( n + 4, (String) i.next() );
     }
}

ResultSet rs = selectFilterStmt.executeQuery( );

while ( rs.next() ) {

    Knowledge_baseVO vo = new Knowledge_baseVO();

    vo.setDoc_key( rs.getInt(1) );
    vo.setDoc_name( rs.getString(2) );
    vo.setCategory( rs.getString(3) );
    vo.setPost_user( rs.getString(4) );
    vo.setDoc_location( rs.getString(5) );
    vo.setLink_doc( rs.getInt(6) ); 
    vo.setBase_doc_key( rs.getInt(7) ); 
    vo.setEntry_date( rs.getString(8) ); 
    vo.setDate_submitted( rs.getString(9) ); 
    vo.setLevel( rs.getInt(10) ); 

    v.add( vo ) ; 

 }

}
catch (SQLException e) {
   System.out.println("SQLException caught in knowledge_baseDAO.getFilteredList): " + e );
   throw new Exception("SQLException caught in knowledge_baseDAO.getFilteredList(): " + e ); 
}
catch (Exception e ) {
   System.out.println("Exception caught in knowledge_baseDAO.getFilteredList(): " + e ); 
   throw new Exception( "Exception caught in knowledge_baseDAO.getFilteredList(): " + e ); 
}

// return the iterator for this collection
return v.iterator();

}

// ----- 



// Constructor 
public Knowledge_baseDAO() throws Exception {

   // create our database helper 
   dbutil = new DBUtil();

   dbMetaData = dbutil.getdbConnection().getMetaData();
   if ( dbMetaData == null ) 
       System.out.println("Null dbMetaData in knowledge_baseDAO.");

   // prepare SQL statements 
   createPreparedStatements();

} 



}
