/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package chapter16;

//import java.util.*;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import db.DBUtil;

public class Message_userDAO {

// hold SQL statements 
private String insertStmtStr;
private String updateStmtStr;
private String deleteStmtStr;
private String selectStmtStr;

// hold prepared statements 
private PreparedStatement  insertStmt;
private PreparedStatement  updateStmt;
private PreparedStatement  deleteStmt;
private PreparedStatement  selectStmt;

// 
DBUtil dbutil;

Message_userVO message_userVO;



// setXX methods 

// convenience methods
public void createPreparedStatements() {



try {

insertStmtStr = "insert into message_user( login, first_name, last_name, location, date_submitted, last_login, pwd ) " + 
                " values (?,?,?,?,?,?,?)";

insertStmt = dbutil.createPreparedStatement( insertStmtStr ); 

updateStmtStr = "update message_user set " + 
                " first_name     = ?, " + 
                " last_name      = ?, " + 
                " location       = ?, " + 
                " date_submitted = ?, " + 
                " last_login     = ?, " + 
                " pwd            = ? " + 
                " where login = ? ";

updateStmt = dbutil.createPreparedStatement( updateStmtStr ); 

deleteStmtStr = "delete from message_user where login = ?";

deleteStmt = dbutil.createPreparedStatement( deleteStmtStr );

selectStmtStr = "select login,  " + 
                " first_name," +
                " last_name," +
                " location, " +
                " date_submitted," +
                " last_login," +
                " pwd " +
                " from message_user where login = ?";

selectStmt = dbutil.createPreparedStatement( selectStmtStr ); 

}
catch (Exception e ) {

 System.out.println("Exception in Message_userDAO.createPreparedStatements(): " + e ); 

}

}

// update the database from our internal value object
public void updateDAO() throws SQLException{

updateStmt.setString(1, message_userVO.getFirst_name() ); 
updateStmt.setString(2, message_userVO.getLast_name() ); 
updateStmt.setString(3, message_userVO.getLocation() ); 
updateStmt.setString(4, message_userVO.getDate_submitted() ); 
updateStmt.setString(5, message_userVO.getLast_login() ); 
updateStmt.setString(6, message_userVO.getPwd() ); 
updateStmt.setString(7, message_userVO.getLogin() ); 

updateStmt.executeUpdate();

}

// insert contents of our internal VO
public void insertDAO() throws SQLException {

insertStmt.setString(1, message_userVO.getLogin() );
insertStmt.setString(2, message_userVO.getFirst_name() );
insertStmt.setString(3, message_userVO.getLast_name() );
insertStmt.setString(4, message_userVO.getLocation() );
insertStmt.setString(5, message_userVO.getDate_submitted() );
insertStmt.setString(6, message_userVO.getLast_login() );
insertStmt.setString(7, message_userVO.getPwd() );

insertStmt.executeUpdate();

}

public void deleteDAO() throws SQLException {

deleteStmt.setString(1, message_userVO.getLogin() ); 

deleteStmt.executeUpdate();

}

// load from the database 
public void loadDAO( String login ) throws Exception {

try {

selectStmt.setString(1, login ); 

ResultSet rs = selectStmt.executeQuery(); 

if ( rs.next() ) { 

    message_userVO.setLogin( rs.getString(1) ); 
    message_userVO.setFirst_name( rs.getString(2) ); 
    message_userVO.setLast_name( rs.getString(3) ); 
    message_userVO.setLocation( rs.getString(4) ); 
    message_userVO.setDate_submitted( rs.getString(5) ); 
    message_userVO.setLast_login( rs.getString(6) ); 
    message_userVO.setPwd( rs.getString(7) ); 
}
else {  // no records found 
    System.out.println("Message_userDAO.loadDAO(): no records found.");
}

}
catch (SQLException e) {
   System.out.println("SQLException caught in Message_userDAO.loadDAO() " + e );
   throw new Exception("Exception caught in Message_userDAO.loadDAO() " + e ); 
}

}

// set internal ValueObject 
public void setVO( Message_userVO vo ) {

    this.message_userVO = null; // assert we currently have one 
    this.message_userVO = vo;

}

// return the internal value object 
public Message_userVO getVO() {
  return this.message_userVO;
}

// constructor 
public Message_userDAO() {

    // create our value object 
    message_userVO = new Message_userVO();

    // create dbutil
    dbutil = new DBUtil();
 
    // create prepared statements 
    createPreparedStatements();

}

public Message_userDAO( javax.servlet.ServletContext context ) {

    // create dbutil
    dbutil = new DBUtil( context );
 
    // create prepared statements 
    createPreparedStatements();



}



}
