/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */

package chapter17;

import java.text.DateFormatSymbols;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.jsp.JspWriter;

public class Cal {


// array for months 
private static final String[] months = new DateFormatSymbols().getMonths();

// array for days of week in string format
private static final String[] days = new DateFormatSymbols().getWeekdays();

// array for day abbreviations 
private static final String daysAbbrev[] = new DateFormatSymbols().getShortWeekdays();

// internal date 
private int month;
private int day; 
private int year; 

private  String printHighLightColor = "grey";
//
private Calendar  mCalendar = null;  // internal calendar 
private JspWriter mOut = null;       // response output stream   

//
private boolean dateStringOnly = false;


// ***************************************************
// Bean methods 


public void setMonth( int month ) {

  // months are 1-based though internal arrays to track months are 0-based 
  this.month = month;
}  


public void setYear( int year ) {
  this.year = year;
}

public void setDay( int day ) {
   this.day = day;
}

public void setHighLightColor ( String color ) {
   this.printHighLightColor = color;
}

public void setDateString( boolean flag ) {
   this.dateStringOnly = flag;
}

public void setCalDate() {
   // set to internal date 
   setCalDate( this.month, this.day, this.year ); 
}

public void setCalDate( int month, int day, int year ) 
{
    setMonth( month );
    setDay( day );
    setYear( year );

    // 0-based months 
    if ( month > 0 ) 
        month--;

    mCalendar = new GregorianCalendar( year, month, day );
}

public void setOut( JspWriter out ) {
    mOut = out; 
}

public void setCalCurrentDate()
{
   mCalendar = new GregorianCalendar();

   setMonth( mCalendar.get( Calendar.MONTH ) + 1 );
   setDay( mCalendar.get(Calendar.DAY_OF_MONTH ));
   setYear( mCalendar.get(Calendar.YEAR) );

}

// getXXXX() methods 


// return the Current date (month day, year) as a string 
public String getCurrentDate() {

if ( mCalendar == null )
    setCalCurrentDate();

return months[ this.month - 1 ] + 
                   "   " + 
                this.day + 
                   "," + 
                   "   " + 
	       this.year; 
}


public String getCurrentDate( String format ) {

if ( mCalendar == null )
    setCalCurrentDate();

if ( format.equals("mm/dd/yy") )
   return ( mCalendar.get( Calendar.MONTH ) + 1 ) + "/" +
            this.day + "/" +  
            mCalendar.get( Calendar.YEAR );
else        // no other formats for now 
   return null;

}

// ----------------------------------------------------------------------
public void printCal( JspWriter out ) throws ServletException {

Calendar cal = mCalendar;
String printAttr;

// set to the first day of the month 
cal.set( Calendar.DAY_OF_MONTH, 1 ); 

// print the month in HTML table format
try {

// print the header 
out.println("<H2> " + 
                   months[ this.month - 1 ] + 
                   "&nbsp;&nbsp;" + 
                   this.day + 
                   "," + 
                   "&nbsp;&nbsp;&nbsp;" + 
		   this.year + 
		   "</H2>" );

out.println("<table border=3>");
//
// print the days of the week
// this is the calendar header 
//
out.print("<tr>");
for ( int n = 1; n <= 7; n++ ) {
    out.print("<td>" + daysAbbrev[n] + "</td>");
}

out.println("</tr>");

// print blanks up to the start day
out.print("<tr>");

//
// this is the day selected. let's highlight it with this display attribute   
//
if (  this.day == 1 ) 
    printAttr = "<td bgcolor=" + '"' + printHighLightColor + '"' + ">";
else
    printAttr = "<td>";

if ( cal.get( Calendar.DAY_OF_WEEK) > 1 ) { 

   for ( int x = 0; x <= ( cal.get( Calendar.DAY_OF_WEEK) - 1); x++ ) {

       // 
       // these are exceptions to just printing the numeric day of the week 
       //
       if ( x < ( cal.get(Calendar.DAY_OF_WEEK )-1) )  
           out.print( "<td>" + "-" + "</td>" );
       else // it's the first day 
       	   out.print( printAttr + "1" + "</td>" );

       if ( x == 6 ) {
          out.println("</tr>");
          out.print("<tr>");
       }
     }
}
else // day_of_week == 1 == 'Sunday'  
    out.print( printAttr + "1" + "</td>" );
   
    //
    // print the day of the month
    //
    printAttr = "<td bgcolor=" + '"' + printHighLightColor + '"' + ">";

for ( int n = 2; n <= cal.getActualMaximum( Calendar.DAY_OF_MONTH ); n++ ) {

    if ( n == this.day ) // this is the day selected. let's highlight it  
        out.print( printAttr + n + "</td>" );
    else 
        out.print( "<td>" + n + "</td>" );

    // print <tr> at end of week
    cal.set( Calendar.DAY_OF_MONTH, n ); 
    if ( cal.get( Calendar.DAY_OF_WEEK) == 7) {
	out.println( "</tr>" ); 
	out.print( "<tr>" );
    }
}

out.println( "</tr>" ); 
out.println("</table>");
}
catch (IOException e) {
   e.printStackTrace();
   throw new ServletException("IOException in Cal.printCal(): " + e.getMessage() ); 
}

catch (Exception e) {
   e.printStackTrace();
   throw new ServletException("Exception in Cal.printCal(): " + e.getMessage() ); 
}

}




}
