 

/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */

package chapter21;

import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import javax.sql.DataSource;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;


public class BlobWriter {

private Connection con;
private PreparedStatement  prepStmt;

public static void main( String args[] ) {

BlobWriter bw = new BlobWriter();

// retrieve the blob and write to the fileoutputstream
if ( args.length > 0 ) 
   bw.getBlob( Integer.parseInt( args[0] ) , System.out );
else  
   bw.getBlob( 801, System.out );

}

// ----------------------------------------------------------------------------
public void getBlob(int movie_id, OutputStream out ) {

try {
  prepStmt.setInt( 1,movie_id );

  writeBlob( out );
}
catch (SQLException e) {
      System.err.println("SQLException in getBlob: " + e.getMessage());
}

}

// ----------------------------------------------------------------------------

private void writeBlob( OutputStream out ) {
int byteCount = 0;
byte[] buffer = new byte[1024*5];

try {

ResultSet rs = prepStmt.executeQuery();
if ( !(rs.next()) ) {
   System.err.println("Blob row not found.");
   return;
}

InputStream input = rs.getBinaryStream( 2 ); 

byteCount = input.read( buffer );
while ( byteCount > 0 ) {

       out.write( buffer );
       byteCount = input.read( buffer );

}

input.close();
out.close();
}
catch( IOException e) {
       System.err.println( "IOException in writeBlob: " + e.getMessage() );
}
catch( SQLException e) {
       System.err.println( "SQLException in writeBlob: " + e.getMessage() );
}


}

// ----------------------------------------------------------------------------
private void prepareStatement() {

try { 
   prepStmt = con.prepareStatement( "select movie_id, movie_image " + 
                                   " from movie_images " +
	              	           " where movie_id = ?" );

}
catch (SQLException e ) {
      System.err.println("SQLException in prepareStatment: " + e.getMessage() );
}

}

// ----------------------------------------------------------------------------

private void  getConnected() {

 getConnected( "movies" );

}

// ----------------------------------------------------------------------------
private void getConnected(String dataSourceName ) {
InitialContext context=null;
String lookupName = null;
try { 

     context = new InitialContext( );
     DataSource ds = (DataSource) context.lookup( dataSourceName );
     con = ds.getConnection();

  }
catch (SQLException e) {
     System.out.println("SQLException caught in getConnected: " + e.getMessage() );
}
catch (NamingException e) {
     System.out.println("NamingException caught in getConnected: " + e.getMessage() + " - " + e );
     System.out.println("DataSource name: " + dataSourceName );
     System.out.println("Resolved Name: " + e.getResolvedName()  );


}
catch (Exception e) {
     System.err.println("Exception caught in getConnected: " + e );
}

}

public BlobWriter() {

  //
  // create a connection to the database 
  // using the default datasource 
  // 
  new BlobWriter( "movies" ); 

}

public BlobWriter( String dataSourceName ) {

  // create a connection to the database 
  getConnected( dataSourceName ); 

  // prepare the sql statement
  prepareStatement();

}

}
