
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */



package chapter6;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLWarning;
import java.sql.SQLException;
import java.sql.DatabaseMetaData;


import java.io.PrintWriter;
import java.io.FileWriter;

import javax.sql.DataSource;
import javax.sql.ConnectionPoolDataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;

public class DemoWarnings {


public static void main( String args[] ) {

Connection con=null;
ResultSet rs = null;
try { 

     //
     // get the initial context 
     //
     InitialContext ctx = new InitialContext(  );

     //
     // get the DataSource from the JNDI name server 
     //
     DataSource  ds   = (DataSource) ctx.lookup("movies");

   if (ds instanceof ConnectionPoolDataSource)
      System.out.println("it's a pooled data source.");

   PrintWriter pw  = ds.getLogWriter();
   if (pw == null) 
      ds.setLogWriter(new PrintWriter( new FileWriter("test.log")) );

   pw  = ds.getLogWriter();
   if ( pw != null ) 
       pw.println("** hello world. ** ");
   pw.flush();

   ds.setLoginTimeout(200);

   // 
   // get the connection from the DataSource
   // 
   con = ds.getConnection( );

   if ( con.getAutoCommit() == true ) 
       System.out.println("Driver is in auto-commit mode.");


//
// check for database warnings 
//
SQLWarning sqlw = con.getWarnings();

//
// warning may be null reference 
//
if (sqlw != null) {
 
   // 
   // iterate through the chain of warnings 
   //
   while ( sqlw != null ) 
         System.out.println("Warning: " + sqlw );
	 sqlw = sqlw.getNextWarning();
   }
else
   System.out.println("There are no warnings.");


   //
   // clear all warnings on the connection 
   // 
   con.clearWarnings();

   // print somewhat useful information 
   System.out.println("catalog is : " + con.getCatalog() );

   DatabaseMetaData dmd = con.getMetaData();

   Statement stmt20 = con.createStatement();
   stmt20.executeUpdate("insert into testit (col2) values('yes')");
   SQLWarning sqlw20 = stmt20.getWarnings();
   if (sqlw20 != null) {
 
     // 
     // iterate through the chain of warnings 
     //
     while ( sqlw20 != null ) {
           System.out.println("Warning: " + sqlw20 );
           System.out.println("Warning getMessage(): " + sqlw20.getMessage() );
 	   sqlw20 = sqlw20.getNextWarning();
     }

}
else { 
	System.out.println("NO warnings from sqlw20 ... " );
}

				
}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in main: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
}
catch (Exception e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

    try {   
        // close the connection 
	if ( con != null )
            con.close(); 
	System.exit(1); 
    }
    catch (SQLException e) {
          e.printStackTrace();
    }
}



}

}
