/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package chapter6;

import java.sql.ResultSet;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class StatementExample2 {


public static void main( String args[] ) {

ResultSet rs    = null;
Connection con  = null;
try { 

     //
     // JNDI startup parameters are stored in the "jndi.properties" file in the 
     // classpath.
     //
     InitialContext ctx = new InitialContext( );

     //
     // get the DataSource from the JNDI name server 
     //
     DataSource ds = (DataSource) ctx.lookup("movies");

     //
     // get the connection from the DataSource
     //
     con = ds.getConnection( );

     //
     // create a Statement class to execute the SQL statement 
     // create two statements from the same connection
     //
     Statement stmt = con.createStatement(); 

     //
     // add a batch of SQL statements 
     //
     stmt.addBatch("insert into users (user_id, first_name, last_name) " + 
                              " values ( 1301, 'Sam', 'Snape')" ); 
     stmt.addBatch("insert into users (user_id, first_name, last_name) " + 
                              " values ( 1402, 'Sal', 'Snake')" ); 
     stmt.addBatch("insert into movies (movie_id, movie_name) " + 
                              " values ( 1501, 'The Evil Null Reference')" ); 
     stmt.addBatch("update movies  " + 
                              " set movie_desc = '** not provided **' " ); 

     // execute the batch and get the results 
     //
     int results[] = stmt.executeBatch();

     //
     // print the results. integer values represents rows updated 
     // 
     for (int n=0;n<results.length;n++)
         System.out.println("Result from query " + n + " = " + results[n] );

				
}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in main: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
}
catch (NamingException e) {
     System.out.println("NamingException in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

    try {   

	if ( con != null )
            con.close(); 
    }
    catch (SQLException e) {
          e.printStackTrace();
    }
}



}

}
