
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package chapter6;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;

public class StatementExample4 {


public static void main( String args[] ) {

Connection con=null;
try { 

     //
     // JNDI startup parameters are stored in the "jndi.properties" file in the 
     // classpath.
     //
     InitialContext ctx = new InitialContext( );

     //
     // get the DataSource from the JNDI name server 
     //
     DataSource ds = (DataSource) ctx.lookup("movies");

     //
     // get the connection from the DataSource
     //
     con = ds.getConnection( );

     //
     // create a Statement class to execute the SQL statement 
     //
     Statement stmt = con.createStatement(); 

     // 
     // get some information about the fetch limits 
     //
     System.out.println( "Fetch size: " + stmt.getFetchSize() );
     System.out.println( "Field size: " + stmt.getMaxFieldSize() );
     System.out.println( "Max rows: " + stmt.getMaxRows() );
     System.out.println( "Query timeout: " + stmt.getQueryTimeout() );

     //
     // tune these parameters 
     //
     stmt.setMaxRows( 500 );
     stmt.setFetchSize( 100 );

     // execute the SQL statement and get the results in a ResultSet 
     ResultSet rs   = stmt.executeQuery("select movie_name, release_date from movies" ); 

     // iterate through the ResultSet, displaying two values 
     // for each row using the getString method  
     // 
     while ( rs.next() ) {
            System.out.println( "******************" + "\n" + 
	                        "movie_name: " + "\t" + rs.getString("movie_name") + "\n" + 
	                        "release_date: " + "\t" + rs.getString("release_date")  ); 
     }
				
}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in main: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
}
catch (NamingException e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

    try {   
        // close the connection 
	if ( con != null )
            con.close(); 
    }
    catch (SQLException e) {
          e.printStackTrace();
    }
}



}

}
