
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package chapter6;

import java.sql.ResultSet;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.SQLException;
import javax.sql.DataSource;
import javax.naming.Context;
import javax.naming.InitialContext;

public class StatementExampleX {


public static void main( String args[] ) {

ResultSet rs    = null;
Connection con  = null;

try { 

     //  
     // Either connect to an existing poolman instance or start poolman here. 
     // This will register JNDI name with the JNDI service provider and initialize pools

     //
     // JNDI startup parameters are stored in the "jndi.properties" file in the 
     // classpath.
     //
     InitialContext ctx = new InitialContext( );

     // get the DataSource from the JNDI name server 
      DataSource ds = (DataSource) ctx.lookup("movies");

     // get the connection from the DataSource
     con =  ds.getConnection( );

     // create a Statement class to execute the SQL statement 
     // create two statements from the same connection
     Statement stmt = con.createStatement(); 

     // 
     // execute a query. can be an update or a select will still 
     // use the same code to process the query 
     //
     String[] queries = {"select * from users", 
                         "select * from movies", 
			 "update users set country = 'USA'", // perform update 
			 "select * from users"};             // view update results

for ( int n = 0; n < queries.length; n++ ) {

    //
    // execute the query 
    //
    System.out.println("\n** Executing query: " + n + " : " + queries[n] );
    boolean isResultSet = stmt.execute( queries[n] );

    //
    // is it a ResultSet ?
    //
    if ( isResultSet ) {
        rs = stmt.getResultSet();
    }

     //
     // print the results whether this is an update or a select  
     // 
     if ( isResultSet ) {  // it's a ResultSet
         System.out.println( "\nSelect Performed - Results: " );

         while ( rs.next() ) { 
	       //
	       // loop thru each column
	       //
	       System.out.println("** Table: " + rs.getMetaData().getTableName(1) );
	        
	       for ( int i = 1; i <= rs.getMetaData().getColumnCount(); i++ ) {      
               System.out.println( "\tColumn: "   + rs.getMetaData().getColumnName(i) + 
				   " - value :\t" + rs.getObject(i) ); // value 
               }
        }

      } 
      else {          // it's an update 
         System.out.println("\n Update Performed - Rows updated: " + stmt.getUpdateCount() ); 
      }

rs =  null; // reset this for the next loop 

} // end for loop 
				
}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in main: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
}
catch (Exception e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

    try {   

	if ( con != null )
            con.close(); 
    }
    catch (SQLException e) {
          e.printStackTrace();
    }
}



}

}
