
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */

package chapter7;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.ResultSet;

import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;

import java.text.DateFormat;


public class PreparedExample2 {

PreparedStatement rentalsbyDate;
PreparedStatement rentalsbyUserID;
PreparedStatement rentalsbyCustName;

DateFormat df = DateFormat.getInstance(); // used to format dates for insert 

public static void main( String args[] ) {

ResultSet rs; 

Connection con=null;
try { 

     PreparedExample2 pe = new PreparedExample2();

     // get the initial context 
     InitialContext ctx = new InitialContext(  );

     // get the DataSource from the JNDI name server 
     DataSource ds = (DataSource) ctx.lookup("movies");

     // get the connection from the DataSource
     con = ds.getConnection( );

     // prepare statements 
     pe.doPrepares( con );

     // clear all existing parameter settings
     pe.clearAllParams();

     // execute a report 
     rs = pe.getRentalsbyCustName( "Smith" );  // get everything for customer id = 2

     // print results 
     if ( rs != null ) {
         pe.printResults( "Rentals by Customer Name: Smith", rs );
     }

     // clear all existing parameter settings
     pe.clearAllParams();

     // execute another report 
     rs = pe.getRentalsbyDate( "2001-01-01" ); // greater than or equal to this date 

     // print results 
     if ( rs != null ) {
         pe.printResults( "Rentals by Date: > 1/1/2001", rs );
     }

     // clear all existing parameter settings
     pe.clearAllParams();

     // execute another report 
     rs = pe.getRentalsbyUserID( 2 );  // get everything for customer id = 2

     // print results 
     if ( rs != null ) {
         pe.printResults( "Rentals by Customer ID: 2", rs );
     }


}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in main: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
}
catch (Exception e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

   if ( con != null ){
     try {
        con.close(); // ** not working right in Poolman 4.0Beta
     }
     catch (SQLException e) {
	     System.out.println("SQLException on close.");
     }
   }
}


} 


void doPrepares( Connection con) {

//
// use instance members to store PreparedStatements
//

try { 

   rentalsbyDate = con.prepareStatement( " select distinct first_name, last_name, state_province, " + 
                                         " movie_name, rental_date "                      + 
					 " from users, movies, user_rentals "             + 
					 " where (users.user_id    = user_rentals.user_id and " + 
					 "        movies.movie_id  = user_rentals.movie_id) and " + 
					 "       user_rentals.rental_date >= ? " ); 
   

   rentalsbyUserID = con.prepareStatement( " select first_name, last_name, state_province, " + 
                                           " movie_name, rental_date "                      + 
					   " from users, movies, user_rentals "             + 
					   " where (users.user_id    = user_rentals.user_id and " + 
					   "        movies.movie_id  = user_rentals.movie_id ) and " + 
					   "       user_rentals.user_id = ? " ); 
   

   rentalsbyCustName = con.prepareStatement( " select distinct first_name, last_name, state_province, " + 
                                             " movie_name, rental_date "                      + 
					     " from users, movies, user_rentals "             + 
					     " where ( users.user_id   = user_rentals.user_id and " + 
					     "         movies.movie_id = user_rentals.movie_id ) and " + 
					     "       users.last_name like ? " ); 

}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in doPrepares(): " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();

}

} 
// ************************************************************************


ResultSet getRentalsbyUserID ( int custID ) {


ResultSet rs = null;

try {

//
// set the customer ID parameter as an integer
//
rentalsbyUserID.setInt(1, custID);
rs = rentalsbyUserID.executeQuery();


}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in getRentalsbyUserID(): " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();

}
finally {
   return rs;
}
}



// ************************************************************************

ResultSet getRentalsbyDate( String rentalDate ) {

ResultSet rs = null;

try {

rentalsbyDate.setDate(1,  java.sql.Date.valueOf( rentalDate ) );
rs = rentalsbyDate.executeQuery();

}

catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in getRentalsbyDate(): " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();

}
finally {
   return rs;
}

}
// ************************************************************************


ResultSet getRentalsbyCustName( String custName ) {

ResultSet rs = null;

try {

//
// set the rental date parameter as a String ... SQL will do conversion 
//
rentalsbyCustName.setString( 1, custName );
rs = rentalsbyCustName.executeQuery();


}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in getRentalsbyCustName(): " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();

}
finally {
   return rs;
}

}

// ************************************************************************
void printResults( String header, ResultSet rs) {

try {

// output a report header 
System.out.println("********************************************");
System.out.println("\n******** " + header.trim() + " ********" );
System.out.println("********************************************");
System.out.println("First \tLast \t State \t Movie \t\t\t\tRental Date ");
System.out.println("----- \t---- \t ----- \t ----- \t\t\t\t----------- ");

// output results 
while ( rs.next() ) {
   System.out.println( rs.getString( "first_name") + "\t" + 
                       rs.getString( "last_name" ) + "\t" + 
                       rs.getString( "state_province" ) + "\t" + 
                       formatString( rs.getString( "movie_name" ), 22) + 
		       "\t\t" + 
     //                  rs.getString( "rental_date" )  ) ; 
                  DateFormat.getDateInstance( DateFormat.LONG ).format( rs.getDate( "rental_date" ) ) 
		       ) ; 
}
System.out.println("********************************************");

//rs.close();

}

catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in printResults(): " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();

}
finally {
}



}

void clearAllParams() {

try { 
//
// call clearParameters on each PreparedStatement object 
// this will clear all existing parameter settings 
//
rentalsbyDate.clearParameters();
rentalsbyUserID.clearParameters();
rentalsbyCustName.clearParameters();
}
catch (SQLException e) {
     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in clearAllParams(): " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();

}



}



// ******************************************************************
String formatString( String s, int size ) {

String blanks = "                         ";   // work buffer of blank characters
String retString = null;

// pad or trim the string to be exactly 'size' in length 

if ( s.length() < size )                                      // pad 
   retString = s + blanks.substring( 0, size - s.length()) ; 
else // we need to create a substring                         // trim
   retString = s.substring( 0, size ); 

return retString;

}





}
