
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */

package chapter7;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import javax.sql.DataSource;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class PreparedExample5 {


public static void main( String args[] ) {

ResultSet rs    = null;
Connection con  = null;
try { 

     // 
     // get the initial naming context 
     // 
     InitialContext ctx = new InitialContext();

     // 
     // get the movies DataSource and get the connection 
     //
     DataSource ds = (DataSource) ctx.lookup("movies");
     con =  ds.getConnection();


     //
     // clear the database of some entries so we can run this more than 
     // once
     //
     Statement stmt = con.createStatement();
     stmt.executeQuery( "delete from users where user_id in (2101,2201,2401)");
     stmt.executeQuery( "delete from movies where movie_id = 2501");

     //
     // add a batch of SQL statements 
     //
     PreparedStatement ps = con.prepareStatement("insert into users (user_id, first_name, last_name) " + 
                                                 " values ( ?, ?, ?)" ); 
     ps.setInt( 1, 2101);             // user_id 
     ps.setString( 2, "Cal" );       // first_name
     ps.setString( 3, "Coder" );     // last_name 

     //
     // add the first batch update
     //
     ps.addBatch();

     ps.setInt( 1, 2201);                      // user_id 
     ps.setString( 2, "Carrie" );             // first_name
     ps.setString( 3, "Coder" );              // last_name

     //
     // add the second batch update
     //
     ps.addBatch();

     ps.setInt( 1, 2401);                      // user_id
     ps.setString( 2, "Sal" );                // first_name
     ps.setString( 3, "Snake" );              // last_name

     //
     // add the third batch update
     //
     ps.addBatch();

     //
     // use the inherited Statement class addBatch
     // these queries can't contain parameter placeholders
     //
     ps.addBatch("insert into movies (movie_id, movie_name) " + 
                 " values ( 2501, 'Trying Again')" ); 

     ps.addBatch("update movies  " + 
                 " set movie_desc = '** not provided **' " ); 

     //
     // execute the batch and get the results 
     // should see 5 results 
     //
     int results[] = ps.executeBatch();

     //
     // print the results. integer values represents rows updated 
     // 
     for (int n=0;n<results.length;n++)
         System.out.println("Result from query " + (n+1) + " = " + results[n] );

				
}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in main: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
}
catch (NamingException e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

    try {   

	if ( con != null )
            con.close(); 
    }
    catch (SQLException e) {
          e.printStackTrace();
    }
}



}

}
