
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package chapter8;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import java.util.Calendar;

public class ResultSetExample2 {


//
// demonstrate updatable resultsets 
//

public static void main( String args[] ) {

// set ourDate to the current date 
java.sql.Date ourDate =  new java.sql.Date( Calendar.getInstance().getTime().getTime() );

ResultSet  rs  = null;
Connection con = null;
try { 

    // 
    // get the initial context 
    //
    InitialContext ctx = new InitialContext();

    //
    // get the DataSource
    //
    DataSource ds = (DataSource) ctx.lookup("movies");

    // 
    // get the connection
    //
    con = ds.getConnection();

     // 
     // create a Statement class to execute the SQL statement 
     // 
     Statement stmt = con.createStatement(); 


// ** 
// with MySQL, we must use a table with a primary key defined, 
// we must select the PK and must only use one table 
// and we must select the columns we are going to update 
// ** 

    //
    // execute the SQL statement and get the results in a ResultSet 
    //
    rs   = stmt.executeQuery("select movie_id, update_date, movie_name, release_date, " + 
                              " category, special_promotion" + 
                              " from movies" + 
			      " where category='Comedy'" + 
			      " order by movie_id" ); 

     // 
     // display the ResultSet type 
     // 
     System.out.println( "\nResultSet type: " + getCursorTypeString( rs.getType()) +  "\n" ); 

     // 
     // iterate through the ResultSet, displaying two values 
     // for each row using the getString method  
     // 
     while ( rs.next() ) {
            System.out.println( "******************" + "\n" + 
	                        "movie_name: "   + "\t" +         rs.getString("movie_name") + "\n" + 
	                        "special_promotion: "  + "\t" +   rs.getString("special_promotion") + "\n" + 
				"update_date: "  + "\t" +         rs.getDate("update_date") +  "\n" + 
	                        "release_date: " + "\t" +         rs.getString("release_date")  ); 

            // 
            // change the update date to today 
            // 
	    rs.updateDate( "update_date", ourDate );  

            // 
	    // change the special_promotion to 5
            // 
	    rs.updateInt( "special_promotion", 5 );  

            // 
	    // update the database row 
            // 
	    rs.updateRow();
     }

     // 
     // perform an insert 
     // 
       
     // 
     // make sure the DB doesn't have this entry 
     // 
     stmt.executeQuery( "delete from movies where movie_id = 5601");

     // 
     // first, set the values 
     // 
     rs.moveToInsertRow();
     rs.updateInt( "movie_id", 5601 );
     rs.updateDate( "update_date", ourDate );
     rs.updateString( "movie_name", "One Last Try Before the Lights Go Down" );
     rs.updateDate( "release_date", ourDate );
     rs.updateString( "category", "Comedy" );
     rs.updateInt( "special_promotion", 0 );

     // 
     // perform the insert 
     // 
     rs.insertRow();

     // 
     // let's examine the results 
     // 
     rs   = stmt.executeQuery("select movie_id, movie_name, update_date, release_date, special_promotion " + 
                              " from movies" + 
			      " where category = 'Comedy'" ); 

     System.out.println( "\n ** After updates: \n" );			        

     while ( rs.next() ) {
            System.out.println( "******************" + "\n" + 
	                        "movie_name: "   +       "\t" +   rs.getString("movie_name") + "\n" + 
	                        "special_promotion: "  + "\t" +   rs.getString("special_promotion") + "\n" + 
				"update_date: "  +       "\t" +   rs.getDate("update_date") +  "\n" + 
	                        "release_date: " +       "\t" +   rs.getString("release_date")  ); 
     }
				
}
catch (SQLException e) {

     // 
     // display SQL specific exception information 
     // 
     System.out.println("*************************" );
     System.out.println("SQLException in main: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();
}
catch (NamingException e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

   try {   
        // 
        // close the connection 
        // 
  	if ( con != null )
              con.close(); 
      }
      catch (SQLException e) {
            e.printStackTrace();
      }
  }


} 

static String getCursorTypeString( int type ) {

if ( type == ResultSet.TYPE_FORWARD_ONLY ) 
     return "Forward Only ResultSet.";

if ( type == ResultSet.TYPE_SCROLL_INSENSITIVE ) 
     return "Scroll Insensitive ResultSet.";

if ( type == ResultSet.TYPE_SCROLL_SENSITIVE ) 
     return "Scroll Sensitive ResultSet.";

return "Unknown";

}

}
