
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package chapter9;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;

public class TransExample3 {


public static void main( String args[] ) {

Connection con=null;
int updateResult = 0;
try { 

     // 
     // get the initial context 
     //
     InitialContext ctx = new InitialContext(  );

     //
     // get the DataSource from the JNDI name server 
     //
     DataSource ds = (DataSource) ctx.lookup("movies");

     //
     // get the connection from the DataSource
     //
     con = ds.getConnection( );

     //
     // create a Statement class to execute the SQL statement 
     //
     Statement stmt = con.createStatement(); 


     // 
     // make sure these entries don't currently exist in the DB
     //
     stmt.executeQuery("delete from users where user_id in (5001,5002,5003)");
     stmt.executeQuery("delete from user_rentals where user_id in (5002,5003)");

     // 
     // turn auto-commit off 
     //
     con.setAutoCommit( false );

     // 
     // we are now in a transaction ... perform a series of updates 
     //


     //
     // ** note, since this example uses MySQL, we don't have the nextval function, so we 
     // ** hardcode the user_id. MySQL also doesn't like the ';' at the end of the statement 
     //

     // let's count updates to ensure transaction integrity 
     int updateCount = 0; 


     updateResult = stmt.executeUpdate( "insert into users (user_id, first_name, last_name, address1," + 
                         " city, state_province, postal_code, country)  " + 
			 " values ( 5001, 'Fred', 'Smith', '2020 Nowhere Lane', 'Newark', 'NJ', '09090', 'US')" );

     // increment update count 
     updateCount += updateResult;

     updateResult = stmt.executeUpdate( "insert into users (user_id, first_name, last_name, address1," + 
                         " city, state_province, postal_code, country)  " + 
			 " values ( 5002, 'Sally', 'Smith', '2020 Nowhere Lane', 'Newark', 'NJ', '09090', 'US')" );

     // increment update count 
     updateCount += updateResult;

     updateResult = stmt.executeUpdate( "insert into users (user_id, first_name, last_name, address1," + 
                         " city, state_province, postal_code, country)  " + 
			 " values ( 5003, 'Harry', 'Henderson', '3030 Milbourne', 'Tuckamarin', 'NJ', '08090', 'US')" );

     // increment update count 
     updateCount += updateResult;


     // user_id = 2 is renting a movie 
     updateResult = stmt.executeUpdate( "insert into user_rentals (user_id, rental_date, movie_id) " + 
                                        " values ( 5002, current_date, 1 ) " );
     // increment update count 
     updateCount += updateResult;

     // user_id = 3 is renting a movie 
     updateResult = stmt.executeUpdate( "insert into user_rentals (user_id, rental_date, movie_id) " + 
                                        " values ( 5003, current_date, 3 ) " );

     // increment update count 
     updateCount += updateResult;

     // 
     // need to perform a commit to save the work 
     // 
     if ( updateCount == 5  ) {
         con.commit();
	 System.out.println("Transactions have been committed.");
     }
     else {
         con.rollback();
	 System.out.println("Transactions have been rolled back. Update count: " + updateCount);
     }

     // 
     // 'finally' block will close the connection 
     // 
				
}
catch (SQLException e) {

     // display SQL specific exception information 
     System.out.println("*************************" );
     System.out.println("SQLException in main: " + e.getMessage() );
     System.out.println("** SQLState: " + e.getSQLState());
     System.out.println("** SQL Error Code: " + e.getErrorCode());
     System.out.println("*************************" );
     e.printStackTrace();

     // 
     // if we are in this block, then an error occurred and 
     // our transaction should not continue 
     //
     try { 
         con.rollback();
	 con.close();
	 con=null; // let the finally block know we closed the connection
     }
     catch (SQLException se) {

           // display SQL specific exception information 
           System.out.println("*************************" );
           System.out.println("SQLException in main. COULD NOT ROLLBACK: " + se.getMessage() );
           System.out.println("** SQLState: " + se.getSQLState());
           System.out.println("** SQL Error Code: " + se.getErrorCode());
           System.out.println("*************************" );
           se.printStackTrace();
     }

}
catch (NamingException e) {
     System.out.println("Exception in main: " + e.getMessage() );
     e.printStackTrace();
}
finally {

    try {   
        // close the connection 
	if ( con != null )
            con.close(); 
	System.exit(1); 
    }
    catch (SQLException e) {
          e.printStackTrace();
    }
}



}

}
