
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package db;

import java.sql.ResultSet;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.Statement;
import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.InitialContext;
import java.io.PrintStream;
import java.io.IOException;

public class DBUtil {

// ** should read this from properties file or init param ** 
String dbURL = 
          "jdbc:postgresql://localhost:5432/knowledgebase;user=puser;password=puser";

//String dbDriverName = Class.forName("com.informix.jdbc.IfxDriver");
String     dbDriverName = "org.postgresql.Driver";
Connection dbConnection = null;
ResultSet  dbResultSet  = null;
Statement  dbStatement  = null;

String dataSourceName; 
InitialContext initContext;

javax.servlet.ServletContext context;

boolean driverLoaded       = false;
boolean Connected          = false; 
boolean StatementCreated   = false;


public DBUtil( ) {

try  {
    //
    // get the default dataSourceName 
    //
    initContext = new InitialContext();
    dataSourceName = (String) initContext.lookup( "java:comp/env/dataSourceName" );

    getConnected();
}
catch (Exception e) {
      System.out.println("Can't connect to DB in DBUtil constructor. Exception caught:" + e.getMessage() );
}

}

// ---------------------------------------------------

public DBUtil( String dataSourceName ) {

try {

   // set the default dataSourceName
   this.dataSourceName = dataSourceName;
   getConnected();
}
catch (Exception e) {
      System.out.println("Can't connect to DB in DBUtil constructor. Exception caught:" + e.getMessage() );
}
}



// ---------------------------------------------------
public DBUtil( javax.servlet.ServletContext context ) {

  // save for logging 
  this.context = context;

}


// ---------------------------------------------------
void loadDriver()  throws Exception { 
	    Class.forName( dbDriverName);
}

public void getConnected()  throws Exception {

try {

     InitialContext ctx = new InitialContext();

     DataSource ds = (DataSource) ctx.lookup( dataSourceName );

     dbConnection = ds.getConnection();

     Connected = true;
}
catch (SQLException e) {
     if ( context != null ) 
         context.log( "SQLException in DBUtil.getConnected: " +  e.getMessage() );
     else
         System.err.println( "SQLException in DBUtil.getConnected: " +  e.getMessage() );

     throw new Exception (  "NamingException in DBUtil.getConnected: " +  e.getMessage() );
}
catch (NamingException e) {
     if ( context != null ) 
         context.log( "NamingException in DBUtil.getConnected: "   + e.getMessage() );
     else
         System.err.println( "NamingException in DBUtil.getConnected: " +  e.getMessage() );

     throw new Exception (  "NamingException in DBUtil.getConnected: " + e.getMessage() );
}
catch (Exception e) {
     if ( context != null ) 
         context.log( "NamingException in DBUtil.getConnected: "  + e.getMessage() );
     else
         System.err.println( "NamingException in DBUtil.getConnected: " +  e.getMessage() );

     throw new Exception (  "NamingException in DBUtil.getConnected: "  + e.getMessage() );

}

}

// ----------------------------------------------------------------------

public void createDBStatement() throws Exception {

try {
     if ( !Connected ) 
         getConnected();

     dbStatement  = dbConnection.createStatement();
} 

catch (SQLException e) {
     if ( context != null ) 
         context.log( "SQLException in DBUtil.getConnected: "  + e.getMessage() );
     else
         System.err.println( "SQLException in DBUtil.getConnected: " +  e.getMessage() );

     throw new Exception (  "NamingException in DBUtil.getConnected: " +   e.getMessage() );
}
catch (NamingException e) {
     if ( context != null ) 
         context.log( "NamingException in DBUtil.getConnected: "  + e.getMessage() );
     else
         System.err.println( "NamingException in DBUtil.getConnected: " +  e.getMessage() );

     throw new Exception (  "NamingException in DBUtil.getConnected: " +  e.getMessage() );
}
catch (Exception e) {
     if ( context != null ) 
         context.log( "NamingException in DBUtil.getConnected: "  + e.getMessage() );
     else
         System.err.println( "NamingException in DBUtil.getConnected: " +  e.getMessage() );

     throw new Exception (  "NamingException in DBUtil.getConnected: " +  e.getMessage() );

}




}

public PreparedStatement createPreparedStatement( String stmt ) throws Exception {

try {
     if ( !Connected ) 
         getConnected();
     return dbConnection.prepareStatement( stmt ); 
} 

catch (SQLException e) {
     if ( context != null ) 
         context.log( "SQLException in DBUtil.getConnected: " + e.getMessage() );
     else
         System.err.println( "SQLException in DBUtil.getConnected: " + e.getMessage() );

     throw new Exception (  "NamingException in DBUtil.getConnected: " + e.getMessage() );
}
catch (NamingException e) {
     if ( context != null ) 
         context.log( "NamingException in DBUtil.getConnected: "  + e.getMessage() );
     else
         System.err.println( "NamingException in DBUtil.getConnected: " +  e.getMessage() );

     throw new Exception (  "NamingException in DBUtil.getConnected: "   + e.getMessage() );
}
catch (Exception e) {
     if ( context != null ) 
         context.log( "NamingException in DBUtil.getConnected: "  + e.getMessage() );
     else
         System.err.println( "NamingException in DBUtil.getConnected: " +   e.getMessage() );

     throw new Exception (  "NamingException in DBUtil.getConnected: " + e.getMessage() );

}



}

public ResultSet executePreparedStatement( PreparedStatement pstmt ) throws Exception {

     if ( !Connected ) 
         getConnected();
     return pstmt.executeQuery();

}

public int executePreparedStmtUpd( PreparedStatement pstmt )  throws Exception{

    if ( !Connected ) 
        getConnected();
    return pstmt.executeUpdate();

}


public ResultSet executeDBQuery( String query ) throws Exception {

    // make sure we are ready to do this

     if ( !Connected )  
	 getConnected();
    
     if ( !StatementCreated ) {
	  createDBStatement();
	  StatementCreated = true;
     }

     // execute query 
     dbResultSet   = dbStatement.executeQuery( query );

     return dbResultSet;

}


public int executeUpdDBQuery( String query ) throws Exception {

    // make sure we are ready to do this
     if ( !Connected )  
	 getConnected();
    
     if ( !StatementCreated ) {
	  createDBStatement();
	  StatementCreated = true;
     }

     // execute query 
     int retVal   = dbStatement.executeUpdate( query );

      // return number of rows updated
      return retVal;

}


// Bean methods 
public ResultSet getdbResultSet() {
	return dbResultSet;
}

public String getdbDriverName() {
	return dbDriverName;
}

public Connection getdbConnection() {
	return dbConnection;
}

public String getDbURL() { 
	return dbURL;
}

public Statement getdbStatement() {
	return dbStatement;
}


public void setdbDriverName ( String dbDN ) {
	this.dbDriverName = dbDN;
	driverLoaded = false; // force new driver load 
}

public void setdbURL ( String dbURL ) {
	this.dbURL = dbURL;
	Connected = false; // force new connection
}

public void setOutputStream( PrintStream Out ) {
}


} // end class 
