
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package db;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.ResultSet;

public class MoviesDAO {

// hold query strings 
private String insertStmtStr;
private String updateStmtStr;
private String deleteStmtStr;
private String selectStmtStr;
private String selectCategoryListStr;

// hold prepared statements 
private PreparedStatement insertStmt;
private PreparedStatement updateStmt;
private PreparedStatement deleteStmt;
private PreparedStatement selectStmt;
private PreparedStatement selectCategoryListStmt;

DBUtil dbutil;

MoviesVO moviesVO;

boolean isEmpty;

public static void main( String args[] ) {


MoviesDAO mdao = new MoviesDAO( "moviesmysql" );

mdao.loadDAO( 1 ); // get movie 1 
mdao.getMoviesVO().dump();

mdao.loadDAO( 2 ); // get movie 1 
mdao.getMoviesVO().dump();

}



// data access methods 
public void createPreparedStatements() {

try {

insertStmtStr = "insert into movies (movie_id, movie_name, release_date) values (?,?,?)";

insertStmt = dbutil.createPreparedStatement( insertStmtStr ); 
   
updateStmtStr = "update movies set movie_name = ? where movie_id = ? ";

updateStmt = dbutil.createPreparedStatement( updateStmtStr );

deleteStmtStr = "delete from movies where movie_id = ?"; 

deleteStmt = dbutil.createPreparedStatement( deleteStmtStr ); 

selectStmtStr = "select * from movies where movie_id = ?";

selectStmt = dbutil.createPreparedStatement( selectStmtStr ); 

}
catch( Exception e) {

  System.out.println("Exception in MoviesDAO.CreatePreparedStatement(): " + e.getMessage() );

}

}
// ---------------------------------------------------

public void updateDAO() throws SQLException {

    updateStmt.setString(1, moviesVO.getMovie_name() ); 
    updateStmt.setInt(2, moviesVO.getMovie_id() ); 

    updateStmt.executeUpdate();

}

public void insertDAO() throws SQLException {

     insertStmt.setInt(1, moviesVO.getMovie_id() ); 
     insertStmt.setString(2, moviesVO.getMovie_name() ); 
     insertStmt.setString(2, moviesVO.getRelease_date() ); 

     insertStmt.executeUpdate();
}
// --------------------------------------------------

public void deleteDAO() throws SQLException {

      deleteStmt.setInt(1, moviesVO.getMovie_id() ); 

      deleteStmt.executeUpdate();

}
// --------------------------------------------------

public void deleteDAO( int movie_id ) throws SQLException {

      deleteStmt.setInt(1, movie_id ); 

      deleteStmt.executeUpdate();

}

// --------------------------------------------------

// refresh values with our internal value 
public void loadDAO() throws SQLException {

    loadDAO( moviesVO.getMovie_id() );

}

// --------------------------------------------------
// load the internal value object with results from the execution of a 
// query with the specified key 
public void loadDAO( int movie_id )  {

try {

// debug 
System.out.println( "loadDAO: setting selectStmt .. " );

 selectStmt.setInt(1, movie_id ); 

 ResultSet rs = selectStmt.executeQuery(); 

// debug 
System.out.println( "loadDAO: setting selectStmt .. " );

if ( rs == null ) 
   System.out.println("MoviesDAO.loadDAO: null ResultSet.");

// now set our internal values to the value of the ResultSet 
if ( rs.next() ) { 
 moviesVO.setMovie_id(          rs.getInt("movie_id") );
 moviesVO.setMovie_name(        rs.getString("movie_name") ); 
 moviesVO.setRelease_date(      rs.getString("release_date") ); 
 moviesVO.setMovie_desc(        rs.getString("movie_desc") ); 
 moviesVO.setUpdate_date(       rs.getString("update_date") ); 
 moviesVO.setCategory(          rs.getString("category") ); 
 moviesVO.setSpecial_promotion( rs.getInt("special_promotion") ); 
 setEmptyFlag( false );
 }
}
catch (SQLException e ) {
 System.out.println("SQLException in MoviesDAO.loadDAO: " + e.getMessage() + " - " + e );
 setEmptyFlag( true );
}

}

// load values from ValueObject
public void loadDAO( MoviesVO moviesVO ) {

// simply grab the reference passed in
// ( this assumes pass by reference - not true if 
//   this is an RMI object )
// 
this.moviesVO = moviesVO;

}


// ------------------------------------

// return a reference to our internal value object 
public MoviesVO getMoviesVO() {

  return moviesVO;

}

// ------------------------------------

private void setEmptyFlag( boolean b ) {
   isEmpty = b; 
}

public boolean isEmpty() {
   return this.isEmpty;
}


// ------------------------------------



// constructors
public MoviesDAO( String dataSourceName ) {

     // create db wrapper
     dbutil = new DBUtil( dataSourceName );

     moviesVO = new MoviesVO();

     // create SQL prepared statements
     createPreparedStatements();
     

}

public MoviesDAO( ) {

       new MoviesDAO( "java:/knowledgebase" );

}


}
