

/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */



package jdbcutil;

import java.sql.SQLException;
import java.sql.Types;
import java.sql.ResultSet;
import java.text.DecimalFormat;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.math.BigDecimal;

public class JDBCTypeMapper {
//
// map JDBC data types to Java types 
//

public static Object getColumnDataString( int col, ResultSet rs ) {
   Object obj  = null;
   int    type = 0;

   try {       
        type = rs.getMetaData().getColumnType( col );

        switch(type) {

             case Types.INTEGER:
             case Types.SMALLINT:
             case Types.TINYINT:
                 obj =  rs.getObject( col ).toString();
                 break;

             case Types.DOUBLE:
             case Types.DECIMAL:
             case Types.NUMERIC:
             case Types.FLOAT:
	     // perform some formatting - limit to 3 decimal digits on output 
		 NumberFormat nf = (DecimalFormat) NumberFormat.getInstance();
		 nf.setMaximumFractionDigits( 3 );
		 obj = nf.format( rs.getDouble( col ));
                 break;

             case Types.BIT:
                 obj = rs.getBoolean( col ) ? "1" : "0";
	         break;

             case Types.DATE:
	          // use the long date format 
		  if ( rs.getDate( col ) != null ) {
                  DateFormat df  = DateFormat.getDateInstance( DateFormat.LONG );
                  obj = df.format( rs.getDate( col ) );  
		  }
		  else {
		  obj = "0000-00-00";
		  }
	         break;

              case Types.CHAR:
              case Types.VARCHAR:
              case Types.LONGVARCHAR:
                 obj = rs.getString( col );
		 break;

               case Types.BLOB:
	       case Types.CLOB:
	         obj = "<BLOB>";
                 break;

               case Types.NULL:
	         obj = " "; // map to blank string
                 break;

             default:
                 obj = rs.getObject( col ).toString();
	         break;
        }

    }
    catch (SQLException e) {
         System.out.println( "SQLException in getColumnData: " + e.getMessage() );
    }

    finally { 
        return obj;
    }

}
// -----------------------------------------------------------------------

    public static Object getColumnData (int col, ResultSet rs) {
    // return column data as correct Java type 
        int   type  = 0;
	Object obj  = null;

        try {
            type = rs.getMetaData().getColumnType( col );

        switch(type) {
              case Types.CHAR:
              case Types.VARCHAR:
              case Types.LONGVARCHAR:
                  obj = (Object) rs.getString( col );
                  break;

              case Types.BIT:
                  obj = (Object) new Boolean( rs.getBoolean( col ));
                  break;

              case Types.TINYINT:
              case Types.SMALLINT:
              case Types.INTEGER:
                  obj = (Object) new Integer( rs.getInt( col ));
                  break;

              case Types.BIGINT:
                  obj = (Object) new Long( rs.getLong( col ));
                  break;

// ** decimal - bigdecimal ?? ** 
              case Types.FLOAT:
              case Types.DOUBLE:
                  obj = (Object) new Double( rs.getDouble( col ));
                  break;

// ** could retrieve the other decimals as bigdecimal ?? ** 
              case Types.NUMERIC:
                  //obj = (Object) rs.getBigDecimal( col );
                  obj = (Object) new Double( rs.getDouble( col ));
		  break;

              case Types.DATE:
                  obj = (Object) rs.getDate( col ); // 
                  break;

               case Types.BLOB:
               case Types.CLOB:
	         obj = "<BLOB>";
		 break;

              default:
                  obj = (Object) rs.getObject( col );
                  break;
        }

	}
        catch (SQLException e) {
            System.out.println( "SQLException in getColumnData: " + e.getMessage() );
        }

	finally {
               return obj;
	}
    }

}
