
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */


package movies.ejb;

import javax.ejb.*;
import java.rmi.*;
import java.io.Serializable;

import java.sql.*;
import javax.sql.*;
import javax.naming.*;

import db.*;
import jdbcutil.*;

public class MoviesEntityBean implements EntityBean {

EJBHome   ejbHome;
Handle    ejbHandle;

//DBUtil    dbutil;

EntityContext context;

MoviesDAO moviesDAO;
MoviesVO  moviesVO;

// 
// return XML formatted string of help desk status 
//
public Integer ejbFindByPrimaryKey( Integer movie_id) throws FinderException {

Integer retVal = null;

try {
       // getGeneralStatus returns a ResultSet which we 
       // will convert/transform to an XML formatted string 
       // 

  //
  // load our DAO using this movie_id 
  //
  if ( moviesDAO == null ) {
       moviesDAO = new MoviesDAO("java:/moviesmysql");
       moviesVO  = new MoviesVO();
   }

   moviesDAO.loadDAO( movie_id.intValue() ); 

  // 
  // if the movie_name has not been set, then no movie 
  // was found for this primary key 
  //
  if ( moviesDAO.getMoviesVO().getMovie_name() != null )  {
       retVal = movie_id;
  }

  }
  catch (Exception e) {
         System.out.println("Exception in findbyPrimaryKey: " + e.getMessage() + e );
  }

  finally {

       //
       // return the primary key 
       //
       return retVal; 
  }

}

//
public MoviesVO getMoviesVO() {
// 
// return the internal Value Object 
//

  return moviesDAO.getMoviesVO();

}


public void setMoviesVO( MoviesVO moviesVO ) {
// 
// set the internal Value Object for the DAO to that of the MoviesVO
//

  moviesDAO.loadDAO( moviesVO );

}




// -------------------------------------------------------
public Integer ejbCreate(int movie_id, String movie_name, String category) throws CreateException {
//
// create a record for this primary key in the permanent data store
//

try {

    //
    // this is a new entity, so we require a new DAO
    //
    moviesDAO = new MoviesDAO("java:/moviesmysql");
    moviesVO  = new MoviesVO();

    //
    // set our elements to the parameter being passed in 
    //
    moviesVO.setMovie_id( movie_id );
    moviesVO.setMovie_name( movie_name );
    moviesVO.setCategory( category );

    //
    // set the DAO to match our Value Object 
    //
    moviesDAO.loadDAO( moviesVO ); 

    //
    // insert into the database 
    //
    moviesDAO.insertDAO();
}
catch (SQLException e) {
    System.out.println("Exception in ejbCreate: " + e.getMessage() );
}
finally {
    return new Integer( movie_id );
}

}


// ------------------------------------------
public void ejbLoad() {
//
// load the entity bean from the database 
//
try {

if ( moviesDAO == null ) {

    //
    // DAO constructor receives name of DataSource to use 
    //
    moviesDAO = new MoviesDAO("java:/moviesmysql");
    moviesVO  = new MoviesVO();

}

    //
    // synch with the database using the primary key value 
    // passed as a parameter 
    //
    moviesDAO.loadDAO( ((Integer) context.getPrimaryKey()).intValue() );
}
catch (Exception e) {
     System.out.println("Exception in ejbLoad: " + e.getMessage() + " - " + e );
}


}

// ------------------------------------------
public void ejbStore() {
//
// store the contents of the Entity Bean in the permanent data store 
//
try {

    //
    // assert our current state contain the correct primary key 
    //
    moviesDAO.updateDAO();

 }
 catch (SQLException e ) {
     System.out.println("Exception in ejbLoad: " + e.getMessage() );
 }

System.out.println("ejbstore...");
}

// -------------------------------------
public void ejbRemove() { 
//
// remove the record in the permanent data store which relates to 
// this entity bean
//
try {

  moviesDAO.deleteDAO( ((Integer) context.getPrimaryKey()).intValue() ); 

}
catch (SQLException e ) {
    System.out.println( "SQLException in ejbRemove: " + e.getMessage() );
}

}

public void ejbPostCreate(int movie_id, String movie_name, String category) throws CreateException {
}

public void ejbActivate() { }
public void ejbPassivate() { }


public void setEntityContext(EntityContext ec) { 
//
// since ejbCreate is only called if creating a new record, this method represents the most likely 
// entry point for the bean - it is called *before* ejbFindByPrimaryKey. There is no guarantee that 
// ejbCreate will be called each time the bean is used (instantiated) by the server.
//
// Therefore, we will perform our initialization here. 
//

    //
    // create the Data Access Object and Value Object 
    //
    if ( moviesDAO == null ) {
        moviesDAO = new MoviesDAO( "java:/moviesmysql" );
	moviesVO  = new MoviesVO();
    }


    //
    // store the session context 
    //
    this.context = ec;

}


public void unsetEntityContext( ) {}

}
