package servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import javax.sql.DataSource;
import java.net.URL;
import org.xml.sax.InputSource;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.naming.Context;
import java.rmi.RemoteException;
import jdbcutil.JDBCXML;
import com.icl.saxon.trax.Result;
import com.icl.saxon.trax.Processor;
import com.icl.saxon.trax.Templates;
import com.icl.saxon.trax.Transformer;
import com.icl.saxon.trax.ProcessorException;

public class ServletExample1 extends HttpServlet {

//
// for Saxon XML transformations
//
private Processor   processor;
private Templates   templates;
private Transformer transformer;

private PreparedStatement pstmt;
private Connection con;

private String   defaultStyleSheet = "file:/lin/home/art/JDBC_Real/Code/Dynamic/rs.xsl";



    public void doGet(HttpServletRequest request,
                      HttpServletResponse response)
        throws IOException, ServletException
    {
        PrintWriter out = response.getWriter();

        out.println("<html>");
        out.println("<head>");

	out.println("<title>Movies Listing</title>");
        out.println("</head>");
        out.println("<body bgcolor=\"white\">");


        try { 

          if ( pstmt == null ) {                          // need to prepare the query 
	      //
	      // get the query from an environment parameter 
	      //
	      String query = getInitParameter( "Query2" );

              //
	      // prepare the query 
	      //
	      pstmt = con.prepareStatement( query );

              //
	      // return an XML encoded string with the results 
	      //
	  }

	  //
	  // execute the prepared query and retrieve the results
	  // results are formatted as an XML document 
	  //
	  String xmlDoc = getQueryResults();

          //
          // transform the XML to the format for our output (an HTML table)
	  //
	  //Transformer transformer = templates.newTransformer();
          transformer.transform(  new InputSource( new StringReader( xmlDoc ) ),
	                          new Result( out ) ); 

           out.println("</body>");
           out.println("</html>");
        }
	catch (Exception e) { 
              log( "Exception in doGet: " + e.getMessage() );
	}
      
      }
// -----------------------------------------------------------------------

public String getQueryResults() {
ResultSet rs = null;
try {
       rs = pstmt.executeQuery( ); 

}
catch (SQLException e) {
       System.err.println("SQLException in ServletExample2.getQueryResults: " + e.getMessage() );
}
finally {
     return JDBCXML.toXMLString( rs );
}

}

// -----------------------------------------------------------------------
public void init( ) {

	  //
	  // get the location of the xslt stylesheet from a context parameter
	  //
	  String styleSheet = getInitParameter( "XsltQuery2" );

try {
	  //
	  // build the URL for the stylesheet 
	  //
	  if ( styleSheet == null ) 
	      styleSheet = defaultStyleSheet;
          else
	      styleSheet = "file:" + styleSheet;

          // 
	  // create the objects we need for the XSL transformation 
	  //
	  processor = Processor.newInstance("xslt");
	  //
	  // get an InputStream from the URL 
          //
	  templates = processor.process(new InputSource( 
	                  new URL( styleSheet ).openStream() ) );

	  transformer = templates.newTransformer();

          //
	  // prepare data access
	  //
          InitialContext init = new InitialContext( );

          //
	  // get the DataSource and create a connection
	  //
	  String dataSourceName = getInitParameter( "data-resource-name" );
	  DataSource ds = (DataSource) init.lookup( dataSourceName );
	  con = ds.getConnection();

}
catch (SQLException e) {
       System.out.println("SQLException in ServletExample2.init: " + e.getMessage() );
}
catch (ProcessorException e) {
       System.out.println("ProcessorException in ServletExample2.init: " + e.getMessage() );
}
catch (RemoteException e) {
       System.out.println("RemoteException in ServletExample2.init: " + e.getMessage() );
}
catch (NamingException e) {
       System.out.println("NamingException in ServletExample2.init: " + e.getMessage() + " - " + e );
}
catch (Exception e) {
       System.out.println("Exception in ServletExample2.init: " + e.getMessage() );
}


}

}



