
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */
package util.blobs;

import java.sql.*;
import javax.sql.*;
import java.io.*;
import javax.naming.*;
import java.util.Hashtable;
import javax.naming.directory.DirContext;

public class BlobLoad {

public static void main( String args[] ) {
String fileName=null;

try {

//
// process the command line arguments  
//
if ( args.length == 3 ) {

    // get the file name where the image is located 
    fileName   =  args[1];

}
else {
    System.out.println( "BlobLoad usage:   BlobLoad <movie_id> <file_name> <image_type> " );
    System.exit(-1);
}

BlobLoad bl = new BlobLoad();

Hashtable env = new Hashtable();

env.put( Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.ldap.LdapCtxFactory" );
env.put( Context.PROVIDER_URL, "ldap://localhost:389" );
env.put( Context.SECURITY_AUTHENTICATION, "simple" );
env.put( Context.SECURITY_CREDENTIALS, "secret" );
env.put( Context.SECURITY_PRINCIPAL, "cn=movies,dc=movies,dc=com" );

InitialContext init = new InitialContext( env );

DirContext ctx = (DirContext) init.lookup( "o=jdbc, o=general-application-objects, dc=movies, dc=com" );

DataSource ds = (DataSource) ctx.lookup( "cn=movies" );

Connection con = ds.getConnection();

PreparedStatement pstmt = con.prepareStatement( "insert into movie_images " + 
                                                " ( movie_id, movie_image, image_type ) " + 
                                                " values( ?,?,? ) " );

// set the movie_id
pstmt.setInt(1, Integer.parseInt( args[0] ));
    
// set the image_type 
pstmt.setString(3, args[2] ); // image type field
//
// Read the graphic image on disk and place in the database 
// as a binary stream
//

//
// column 2 is the Blob column
//
pstmt.setBinaryStream( 2, new FileInputStream( fileName ), 0 ); // 0 - use file length

//
// update the database - driver will read and load the blob
//
System.out.println( "Loading Blob ... " ); 
pstmt.executeUpdate();
System.out.println( "Blob load complete ... " ); 


}
catch (Exception e) {
 System.out.println("Exception in main: " + e.getMessage() + " - " + e);
}

}



// -----------------------------------------------------------------------
public void loadBlob( PreparedStatement pstmt, int pos, String fileName ) {
// load a blob from the designated file using the prepared statement with 
// the blob at the designated position

try {

   InputStream stream = new FileInputStream( fileName );

   pstmt.setBinaryStream( pos, stream, 0 );

}
catch (SQLException e) { 
      System.out.println("SQLException in loadBlob: " + e.getMessage() );
}
catch (FileNotFoundException e) {
      System.out.println("FileNotFoundException in loadBlob: " + e.getMessage() );
}

}
// -----------------------------------------------------------------------
public void outputBlob( ResultSet rs, int pos, String fileName ) {

// buffer size ?? 
byte[] buffer = new byte[1024*5];
int byteCount = 0; 

try {

//
// get the Binary Stream (InputStream) for the Blob column
//
InputStream input = rs.getBinaryStream( pos ); 

//
// create and OutputStream using the fileName parameter
//
FileOutputStream output = new FileOutputStream( fileName );

// 
// write the Blob to the disk file 
//
byteCount = input.read( buffer );
while ( byteCount > 0 ) {

       // 
       // output bytes 
       //
       output.write( buffer );

       //
       // get the next buffer 
       //
       byteCount = input.read( buffer );

}

//
// close file handles
//
input.close();
output.close();
}
catch (FileNotFoundException e) {
       System.out.println("FileNotFoundException in outputBlob: " + e.getMessage() );
}
catch (IOException e) {
       System.out.println("IOException in outputBlob: " + e.getMessage() );

}
catch (SQLException e) {
       System.out.println("SQLException in outputBlob: " + e.getMessage() );

}


}
// -----------------------------------------------------------------------


}
