
/*
 * Copyright (c) 2002 Prentice Hall
 * 
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. 
 *
 * PRENTICE HALL MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. PRENTCE HALL SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Author: Art Taylor  
 */
package util.blobs;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.DriverManager;

import javax.sql.*;
import javax.naming.*;
import java.io.*;


/** 
 *
 */
public class TestBlob extends Object {

   Connection Conn = null;
    ResultSet RS = null;
    Statement Stmt = null;

    static String DBUrl = "jdbc:mysql://127.0.0.1:3306/test";

    public static void main(String[] Args) throws Exception
    {
	if (Args.length > 0) {
		DBUrl = Args[0];
	}

	TestBlob  B = new TestBlob();

    }

    public TestBlob() throws Exception
    {
	try {
	   //Class.forName("org.gjt.mm.mysql.Driver").newInstance();
	   // Conn = DriverManager.getConnection(DBUrl);

           System.out.println("getting initial context ... ");
	   InitialContext ctx = new InitialContext();
	   DataSource ds = (DataSource) ctx.lookup("moviesmysql");
	   Conn = ds.getConnection();

           System.out.println("obtained datasource. creating statement ... ");

	   Stmt = Conn.createStatement();

	   System.out.print("Create test data: ");
	   boolean create_ok = createTestData();
	   System.out.println(create_ok ? "passed" : "failed");
	   readTestData();
	}
	catch (SQLException E) {
	    throw E;
	}
	finally {
	    if (RS != null) {
		try {
		    RS.close();
		}
		catch (SQLException SQLE) {}
	    }
	    
	    if (Stmt != null) {
		try {
		    Stmt.close();
		}
		catch (SQLException SQLE) {}
	    }

	    if (Conn != null) {
		try {
		    Conn.close();
		}
		catch (SQLException SQLE) {}
	    }
	}
    }


  void readTestData() {
  try { 

  InputStreamReader reader = null;
  InputStream stream = null;


  ResultSet rs = Stmt.executeQuery( "select blobdata from BLOBTEST" );


  if  ( rs.next() ) {
        stream = rs.getAsciiStream( 1 );
  }

  if ( rs.getMetaData().getCatalogName(1) == null )
       System.out.println("catalog name is null.");

  if ( rs.getMetaData().getSchemaName(1) == null )
       System.out.println("schema name is null.");
  
  BufferedReader r = new BufferedReader( new InputStreamReader( stream ) ); 

   System.out.println("Outputting blob ... " );
   if ( r != null )  { 
       String buffer= r.readLine(); 
       for( int n = 1; n < 50; n++ ) {
              System.out.println("n: " + n + " - value: " + buffer + "\n"); 
	      buffer = r.readLine();
	      if ( buffer == null ) 
	          break;
      }
   }
  }
  catch (Exception e) {
       System.out.println("Exception in readTestData: " + e.getMessage() );
       e.printStackTrace();
  }

  }

// ------------------------------------------------

    private boolean createTestData() throws java.sql.SQLException
    {
	try {
	    
	    //
	    // Catch the error, the table might exist
	    //
	    
	    try {
		Stmt.executeUpdate("DROP TABLE BLOBTEST");
	    }
	    catch (SQLException SQLE) {}
	    
	    Stmt.executeUpdate("CREATE TABLE BLOBTEST (pos int PRIMARY KEY auto_increment, blobdata LONGBLOB)");
	    
            //byte[] testBlob = new byte[128 * 1024]; // 128k blob
	    String s = "now this is a blob. this is a great big blob.\n";
	    for ( int n = 1; n <= 200; n++ )
	         s =  s + "now this is a blob. this is a great big blob.\n";

            byte[] testBlob = s.getBytes();

              PreparedStatement pstmt = Conn.prepareStatement("INSERT INTO BLOBTEST(blobdata) VALUES (?)");
              
              pstmt.setBytes(1, testBlob);
              
              pstmt.execute();

	      int rowsUpdated = pstmt.getUpdateCount();
              
              System.out.println("Updated " + rowsUpdated + " row(s) with byte[] data");

	      pstmt.clearParameters();

	      java.io.ByteArrayInputStream bIn = new java.io.ByteArrayInputStream(testBlob);

              pstmt.setBinaryStream(1, bIn, 0);
              
              pstmt.execute();
              
	      rowsUpdated = pstmt.getUpdateCount();
              System.out.println("Updated " + rowsUpdated + " row(s) with binary stream data");

	      pstmt.clearParameters();

	      System.out.println("Testing parameter check...");

	      boolean passed = false;

	      try 
	      {
		   pstmt.execute();
	      }
	     catch (SQLException sqlEx) {
		if (sqlEx.getMessage().equals("No value specified for parameter 1")) {
			passed = true;
		}
		else {
			sqlEx.printStackTrace();
		}
	     }

	     System.out.println((passed ? "Passed" : "Failed"));
                
                
	}
	catch (SQLException E) {
	    E.printStackTrace();
	    return false;
	}

	return true;
    }
  
}
